import unittest
import sys
from pyxb.utils.fac import *
from pyxb.utils import six
from pyxb.utils.six.moves import xrange

class TestFAC (unittest.TestCase):
    a = Symbol('a')
    b = Symbol('b')
    c = Symbol('c')
    aOb = Choice(a, b)
    aTb = Sequence(a, b)
    a2 = NumericalConstraint(a, 2, 2)
    bTc = Sequence(b, c)
    a2ObTc = Choice(a2, bTc)
    aXb = All(a, b)
    ex = NumericalConstraint(a2ObTc, 3, 5)

    def testSymbol (self):
        self.assertEqual('a', self.a.metadata)
        au = self.a.buildAutomaton()
        cfg = Configuration(au)
        self.assertFalse(cfg.isAccepting())
        cfg.step('a')
        self.assertTrue(cfg.isAccepting())
        cfg.reset()
        self.assertFalse(cfg.isAccepting())
        self.assertRaises(AutomatonStepError, cfg.step, 'b')

    def testNumericalConstraint (self):
        self.assertEqual(self.a2ObTc, self.ex.term)
        self.assertEqual(3, self.ex.min)
        self.assertEqual(5, self.ex.max)

    def testBasicStr (self):
        self.assertEqual('a', str(self.a))
        self.assertEqual('b', str(self.b))
        self.assertEqual('a+b', str(self.aOb))
        self.assertEqual('a.b', str(self.aTb))
        self.assertEqual('&(a,b)', str(self.aXb))
        x = Choice(self.b, self.aTb)
        self.assertEqual('b+a.b', str(x))
        x = Sequence(self.a, self.aOb)
        self.assertEqual('a.(a+b)', str(x))
        x = NumericalConstraint(self.a2ObTc, 3, 5)
        self.assertEqual('(a^(2,2)+b.c)^(3,5)', str(x))

    def testNullable (self):
        x = NumericalConstraint(self.a, 0, 1)
        self.assertTrue(x.nullable)
        self.assertFalse(self.a.nullable)
        self.assertFalse(self.aOb.nullable)
        self.assertFalse(self.aTb.nullable)
        self.assertFalse(self.aXb.nullable)
        x = NumericalConstraint(self.a, 1, 4)
        self.assertFalse(x.nullable)

    def testFirst (self):
        null_position = frozenset([()])
        p0 = frozenset([(0,)])
        p1 = frozenset([(1,)])
        p0or1 = frozenset(set(p0).union(p1))
        self.assertEqual(null_position, self.a.first)
        for p in self.a.first:
            self.assertEqual(self.a, self.a.posNodeMap[p])
        self.assertEqual(p0or1, self.aOb.first)
        self.assertEqual(p0, self.aTb.first)
        for p in self.aTb.first:
            self.assertEqual(self.a, self.aTb.posNodeMap[p])
        rs = set()
        for p in self.a2ObTc.first:
            rs.add(self.a2ObTc.posNodeMap[p])
        self.assertEqual(frozenset([self.a, self.b]), rs)

    def testLast (self):
        null_position = frozenset([()])
        p0 = frozenset([(0,)])
        p1 = frozenset([(1,)])
        p0or1 = frozenset(set(p0).union(p1))
        self.assertEqual(null_position, self.a.last)
        self.assertEqual(p0or1, self.aOb.last)
        self.assertEqual(p1, self.aTb.last)
        rs = set()
        for p in self.a2ObTc.last:
            rs.add(self.a2ObTc.posNodeMap[p])
        self.assertEqual(frozenset([self.a, self.c]), rs)

    def testWalkTermTree (self):
        pre_pos = []
        post_pos = []
        set_sym_pos = lambda _n,_p,_a: isinstance(_n, Symbol) and _a.append(_p)
        self.ex.walkTermTree(set_sym_pos, None, pre_pos)
        self.ex.walkTermTree(None, set_sym_pos, post_pos)
        self.assertEqual(pre_pos, post_pos)
        self.assertEqual([(0,0,0),(0,1,0),(0,1,1)], pre_pos)

    def testCounterPositions (self):
        self.assertEqual(frozenset([(), (0,0)]), self.ex.counterPositions)

    def testFollow (self):
        m = self.a.follow
        self.assertEqual(1, len(m))
        self.assertEqual([((), frozenset())], list(six.iteritems(m)))

    def testValidateAutomaton (self):
        a = Symbol('a')
        x = Sequence(a, a)
        if sys.version_info[:2] >= (2, 7):
            with self.assertRaises(InvalidTermTreeError) as cm:
                x.buildAutomaton()
            self.assertEqual(cm.exception.parent, x)
            self.assertEqual(cm.exception.term, a)
        else:
            self.assertRaises(InvalidTermTreeError, x.buildAutomaton)

    def testUpdateApplication (self):
        cc = CounterCondition(0, 1)
        ui = UpdateInstruction(cc, True)
        values = { cc : 0 }
        self.assertTrue(ui.satisfiedBy(values))
        ui.apply(values)
        self.assertEqual(values[cc], 1)
        if sys.version_info[:2] >= (2, 7):
            with self.assertRaises(UpdateApplicationError) as cm:
                ui.apply(values)
            self.assertEqual(cm.exception.update_instruction, ui)
            self.assertEqual(cm.exception.values, values)
        else:
            self.assertRaises(UpdateApplicationError, ui.apply, values)

    def testInternals (self):
        #print self.ex.facToString()
        au = self.ex.buildAutomaton()
        #print str(au)

    def testAutomaton (self):
        au = self.ex.buildAutomaton()
        cfg = Configuration(au)
        for c in 'aabcaa':
            cfg.step(c)
        self.assertTrue(cfg.isAccepting())

    def testAllConstruction (self):
        tt = All(Symbol('a'), Symbol('b'))
        au = tt.buildAutomaton()
        self.assertEqual(1, len(au.states))
        st = next(iter(au.states))
        self.assertTrue(st.isUnorderedCatenation)

    # Example from Kilpelainen & Tuhkanen, "Towards Efficient
    # Implementation of XML Schema Content Models"
    def testKT2004 (self):
        a = Symbol('a')
        x = NumericalConstraint(Symbol('b'), 0, 1)
        x = NumericalConstraint(Sequence(x, Symbol('c')), 1, 2)
        x = Sequence(NumericalConstraint(Symbol('a'), 0, 1), x, Choice(Symbol('a'), Symbol('d')))
        x = NumericalConstraint(x, 3, 4)
        cfg = Configuration(x.buildAutomaton())
        for word in ['cacaca', 'abcaccdacd']:
            cfg.reset()
            for c in word:
                cfg = cfg.step(c)
            self.assertTrue(cfg.isAccepting())
        for word in ['caca', 'abcaccdac']:
            cfg.reset()
            for c in word:
                cfg = cfg.step(c)
            self.assertFalse(cfg.isAccepting())
        word = list('ad')
        cfg.reset()
        cfg = cfg.step(word.pop(0))
        try:
            cfg = cfg.step(word.pop(0))
            self.fail("Expected recognition error")
        except AutomatonStepError as e:
            self.assertEqual(e.symbol, 'd')
            self.assertEqual(frozenset(e.acceptable), frozenset(['c', 'b']))
        except Exception as e:
            self.fail("Unexpected exception %s" % (e,))

    # Example from email by Casey Jordan to xmlschema-dev mailing list
    # 20100810: http://lists.w3.org/Archives/Public/xmlschema-dev/2010Aug/0008.html
    # This expression is non-deterministic, but at the end only one path is
    # accepting.
    def testCJ2010 (self):
        x = NumericalConstraint(Symbol('b'), 1, 2)
        x = NumericalConstraint(Choice(x, Symbol('c')), 2, 2)
        x = Sequence(Symbol('a'), x, Symbol('d'))
        cfg = Configuration(x.buildAutomaton())
        word = list('abbd')
        cfg = cfg.step(word.pop(0))
        cfg = cfg.step(word.pop(0))
        try:
            cfg = cfg.step(word.pop(0))
            self.fail('Expected nondeterminism exception')
        except NondeterministicSymbolError as e:
            word.insert(0, e.symbol)
        mcfg = MultiConfiguration(cfg)
        mcfg = mcfg.step(word.pop(0))
        mcfg = mcfg.step(word.pop(0))
        accepting = mcfg.acceptingConfigurations()
        self.assertEqual(1, len(accepting))

    def testDeepMulti (self):
        # Verify multiconfig works when non-determinism is introduced
        # in a subconfiguration
        x = NumericalConstraint(Symbol('b'), 1, 2)
        x = NumericalConstraint(Choice(x, Symbol('c')), 2, 2)
        mx = Sequence(Symbol('a'), x, Symbol('d'))
        ax = All(mx, Symbol('e'), NumericalConstraint(Symbol('f'), 0, 1))
        topcfg = Configuration(ax.buildAutomaton())
        word = list('abbde')
        cfg = topcfg.step(word.pop(0))
        # Descended into sub-automaton
        self.assertNotEqual(cfg, topcfg)
        cfg = cfg.step(word.pop(0))
        try:
            cfg = cfg.step(word.pop(0))
            self.fail('Expected nondeterminism exception')
        except NondeterministicSymbolError as e:
            word.insert(0, e.symbol)
        mcfg = MultiConfiguration(cfg)
        mcfg = mcfg.step(word.pop(0))
        mcfg = mcfg.step(word.pop(0))
        # NB: buildAutomaton may not preserve term order
        self.assertEqual(frozenset(mcfg.acceptableSymbols()), frozenset(['e', 'f']))
        accepting = mcfg.acceptingConfigurations()
        self.assertEqual(0, len(accepting))
        mcfg = mcfg.step('f')
        accepting = mcfg.acceptingConfigurations()
        self.assertEqual(0, len(accepting))
        self.assertEqual(mcfg.acceptableSymbols(), [ 'e' ])
        mcfg = mcfg.step('e')
        accepting = mcfg.acceptingConfigurations()
        self.assertEqual(1, len(accepting))

    def testSubAcceptMulti (self):
        a = NumericalConstraint(Symbol('a'), 0, 1)
        b = Symbol('b')
        ax = All(a, b)
        mcfg = MultiConfiguration(Configuration(ax.buildAutomaton()))
        word = list('a')
        mcfg = mcfg.step(word.pop(0))
        acc = mcfg.acceptingConfigurations()
        self.assertEqual(0, len(acc))

    # Example from page 2 of Kilpelainen "Checking Determinism of XML
    # Schema Content Models in Optimal Time", IS preprint 20101026
    # ("K2010") Note that though the paper states this RE is
    # deterministic, it's not in the sense that are multiple paths
    # recognizing the same word.
    def testK2010a (self):
        t = NumericalConstraint(Symbol('a'), 2, 3)
        t = NumericalConstraint(Choice(t, Symbol('b')), 2, 2)
        ex = Sequence(t, Symbol('b'))
        L = [ 'aaaab', 'aaaaab', 'aaaaaab', 'aabb', 'aaabb', 'baab', 'baaab', 'bbb' ]
        cfg = Configuration(ex.buildAutomaton())
        for word in L:
            cfg.reset()
            mcfg = MultiConfiguration(cfg)
            for c in word:
                mcfg.step(c)
            accepting = mcfg.acceptingConfigurations()
            if word in ('aaaaab',):
                self.assertEqual(2, len(accepting))
            else:
                self.assertEqual(1, len(accepting), 'multiple for %s' % (word,))

    # The MPEG-7 example from page 3 of K2010
    def testK2010b (self):
        def makeInstance (num_m, num_reps):
            s = []
            while 0 < num_reps:
                num_reps -= 1
                s.append('t')
                s.append('m' * num_m)
            return ''.join(s)

        m = NumericalConstraint(Symbol('m'), 2, 12)
        ex = NumericalConstraint(Sequence(Symbol('t'), m), 0, 65535)
        cfg = Configuration(ex.buildAutomaton())
        self.assertTrue(cfg.isAccepting())
        cfg = cfg.step('t')
        self.assertFalse(cfg.isAccepting())
        cfg = cfg.step('m')
        self.assertFalse(cfg.isAccepting())
        cfg = cfg.step('m')
        self.assertTrue(cfg.isAccepting())
        cfg = cfg.step('t')
        self.assertFalse(cfg.isAccepting())
        for _ in xrange(12):
            cfg = cfg.step('m')
        self.assertTrue(cfg.isAccepting())
        self.assertRaises(UnrecognizedSymbolError, cfg.step, 'm')

    # Example from page 6 of K2010.  This is the "nondeterministic"
    # expression similar to the "deterministic" one of testK2010a.
    # From the perspective of this implementation, there is no
    # difference.
    def testK2010c (self):
        t = NumericalConstraint(Symbol('a'), 1, 2)
        t = NumericalConstraint(Choice(t, Symbol('b')), 2, 2)
        ex = Sequence(t, Symbol('b'))
        L = [ 'aab', 'aaab', 'abb', 'aabb', 'bbb' ]
        cfg = Configuration(ex.buildAutomaton())
        for word in L:
            cfg.reset()
            mcfg = MultiConfiguration(cfg)
            for c in word:
                mcfg.step(c)
            accepting = mcfg.acceptingConfigurations()
            if word in ('aaab',):
                self.assertEqual(2, len(accepting))
            else:
                self.assertEqual(1, len(accepting), 'multiple for %s' % (word,))
        Lbar = [ 'aa', 'bb' ]
        for word in Lbar:
            cfg.reset()
            mcfg = MultiConfiguration(cfg)
            for c in word:
                mcfg.step(c)
            self.assertEqual(0, len(mcfg.acceptingConfigurations()), 'accepting %s' % (word,))
        cfg.reset()
        mcfg = MultiConfiguration(cfg)
        mcfg.step('a')
        mcfg.step('b')
        self.assertRaises(UnrecognizedSymbolError, mcfg.step, 'a')

    def testExpandAll (self):
        a = Symbol('a')
        b = Symbol('b')
        c = Symbol('c')
        all = All.CreateTermTree(a, b, c)
        import itertools
        cfg = Configuration(all.buildAutomaton())
        for word in itertools.permutations('abc'):
            cfg.reset()
            for c in word:
                cfg.step(c)
            self.assertTrue(cfg.isAccepting())
        cfg.reset()
        cfg.step('a')
        cfg.step('b')
        self.assertFalse(cfg.isAccepting())

    def testTransitionChain (self):
        cc1 = CounterCondition(0, 1)
        cc2 = CounterCondition(3, None)
        psi = frozenset([UpdateInstruction(cc1, False), UpdateInstruction(cc2, True)])
        s1 = State('1', True)
        s2 = State('2', False)
        x1 = Transition(s1, psi)
        x2 = Transition(s2, [UpdateInstruction(cc2, False)])
        x1b = x1.chainTo(x2)
        self.assertNotEqual(x1, x1b)
        self.assertEqual(id(x1.updateInstructions), id(x1b.updateInstructions))
        self.assertEqual(x1.nextTransition, None)
        self.assertEqual(x1b.nextTransition, x2)

    def testTransitionLayers (self):
        a1 = All(NumericalConstraint(Symbol('a'), 0, 1), Symbol('b'), NumericalConstraint(Symbol('c'), 0, 1))
        a2 = All(Symbol('d'), NumericalConstraint(Symbol('e'), 0, 1), Symbol('f'))
        tt = NumericalConstraint(Sequence(NumericalConstraint(a1, 0, 1), NumericalConstraint(a2, 0, 1), Symbol('l')), 0, 3)
        au = tt.buildAutomaton()
        topcfg = Configuration(au)
        cfg = topcfg.step('b')
        cfg = cfg.step('a')
        cfg = cfg.step('e')
        topcfg.reset()
        cfg = topcfg.step('a')
        # Can't move to 'e' until the required component 'b' of a1 has
        # been provided.  'c' is also permitted.
        try:
            cfg = cfg.step('e')
            self.fail('Expected recognition error')
        except AutomatonStepError as e:
            self.assertEqual(e.symbol, 'e')
            # NB: buildAutomaton may not preserve term order
            self.assertEqual(frozenset(e.acceptable), frozenset(['c', 'b']))
        except Exception as e:
            self.fail('Unexpected exception %s' % (e,))
        cfg = cfg.step('b')
        cfg = cfg.step('e')

    def testAllTree (self):
        a1 = All(Symbol('a'), Symbol('b'), Symbol('c'))
        a2 = All(Symbol('d'), Symbol('e'), Symbol('f'))
        ex = Sequence(NumericalConstraint(Symbol('f'), 0, 1), a1, NumericalConstraint(a2, 0, 1), Symbol('l'))
        # print ex
        # f^(0,1).&(a,b,c).&(d,e,f)^(0,1).l
        au = ex.buildAutomaton()
        cfg = Configuration(au)
        for word in ['fabcl', 'fcabl']:
            cfg.reset()
            for c in word:
                cfg = cfg.step(c)
            self.assertTrue(cfg.isAccepting())

    def testNonAllTree (self):
        a1 = Symbol('a')
        a2 = Symbol('d')
        ex = Sequence(NumericalConstraint(Symbol('f'), 0, 1), a1, NumericalConstraint(a2, 0, 1), Symbol('l'))
        # f?ad?l
        au = ex.buildAutomaton()
        cfg = Configuration(au)
        # This checks that the transition from a can jump over the d and find the l.
        for word in ['fal', 'fadl']:
            cfg.reset()
            for c in word:
                cfg = cfg.step(c)
            self.assertTrue(cfg.isAccepting())

    def testFilterXit (self):
        # This models part of the content of time-layoutElementType in
        # the DWMLgen schema from NDFD.  The initial state is reached
        # through 's'.  The state can be re-entered on 's' in two
        # ways: a repetition of the 's' term, and a follow past a
        # trivial occurrence of the 'e' term to another instance of
        # ex.  This produces a state that has two identical
        # transitions, introducing non-determinism unnecessarily.
        # Make sure we filter that.
        s = NumericalConstraint(Symbol('s'), 1, None)
        e = NumericalConstraint(Symbol('e'), 0, None)
        ex = NumericalConstraint(Sequence(s, e), 1, None)
        au = ex.buildAutomaton()
        cfg = Configuration(au)
        cfg = cfg.step('s')
        self.assertEqual(1, len(cfg.candidateTransitions('s')))

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
from __future__ import print_function
import pyxb.bundles.common.xhtml1 as xhtml
import pyxb.utils.domutils

pyxb.utils.domutils.BindingDOMSupport.SetDefaultNamespace(xhtml.Namespace)

head = xhtml.head(title='A Test Document')
body = xhtml.body()
body.append(xhtml.h1('Contents'))
body.append(xhtml.p('''Here is some text.

It doesn't do anything special.'''))

p2 = xhtml.p('Here is more text.  It has ',
             xhtml.b('bold'),
             ' and ',
             xhtml.em('emphasized'),
             ' content with ',
             xhtml.b('more bold'),
             ' just to complicate things.')
body.append(p2)

# Verify we have two b's and an em
assert 2 == len(p2.b)
assert 1 == len(p2.em)

# Generate the document and externally verify that the em is between the two bs.
doc = xhtml.html(head, body)
try:
    xmls = doc.toDOM().toprettyxml()
except pyxb.ValidationError as e:
    print(e.details())
    raise
open('genout.xhtml', 'w').write(xmls)



# -*- coding: shift_jis -*-
#
# Validate the Japanese GML bindings

from __future__ import print_function, unicode_literals
import os.path
import pyxb.utils.saxutils
import fgd_gml
import unittest

# Need this to override parse from default expat, which can't handle
# Japanese encodings.  libxml2 can.  It is not necessary to explicitly
# reference libxml2 anywhere else, although this test does so to ensure
# the test doesn't fail just because libxml2 is not installed.
pyxb.utils.saxutils.SetCreateParserModules(['drv_libxml2'])

class ExampleUnicode_JP (unittest.TestCase):

    def tryit (self, path):
        # Make sure we find the files, for tests where we are not
        # in the examples/unicode_jp directory
        path = os.path.join(os.path.dirname(__file__), path)
        xmls = open(path).read()
        instance = fgd_gml.CreateFromDocument(xmls)
        for name in instance.name:
            return name.value()
        return None

    shortPass = False
    def setUp (self):
        try:
            import drv_libxml2
        except ImportError:
            print('WARNING: libxml2 not installed, test not valid')
            self.shortPass = True
            return
        self.shift_jis = self.tryit('data/shift_jis/FG-GML-13-RailCL25000-20080331-0001.xml')

    def testISO_2022_JP (self):
        if not self.shortPass:
            name = self.tryit('data/iso-2022-jp/FG-GML-13-RailCL25000-20080331-0001.xml')
            self.assertEqual(self.shift_jis, name)

    def testEUC_JP (self):
        if not self.shortPass:
            name = self.tryit('data/euc-jp/FG-GML-13-RailCL25000-20080331-0001.xml')
            self.assertEqual(self.shift_jis, name)

    def testUTF_8 (self):
        if not self.shortPass:
            name = self.tryit('data/utf-8/FG-GML-13-RailCL25000-20080331-0001.xml')
            self.assertEqual(self.shift_jis, name)

    def testTransliteration (self):
        instance = fgd_gml.hyouji_kubun_rekkyo_gata('•\Ž¦')
        self.assertEqual(instance, fgd_gml.hyouji_kubun_rekkyo_gata.hyouji)

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.binding.datatypes as xs
import pyxb.binding.basis
import pyxb.utils.domutils
from pyxb.utils import six

import os.path
xsd='''
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
  <xs:element name="iopt" nillable="true" type="xs:integer"/>
</xs:schema>
'''

#open('schema.xsd', 'w').write(xsd)
code = pyxb.binding.generate.GeneratePython(schema_text=xsd)
#open('code.py', 'w').write(code)
#print code

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

class TestTrac_0058 (unittest.TestCase):
    def testRoundTrip (self):
        xmlt = six.u('<iopt xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:nil="true"></iopt>')
        xmld = xmlt.encode('utf-8')
        instance = CreateFromDocument(xmlt)
        self.assertTrue(instance._isNil())
        self.assertEqual(0, instance)
        self.assertEqual('', instance.xsdLiteral())
        self.assertEqual(instance.toxml("utf-8", root_only=True), xmld)

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
from pyxb.exceptions_ import *
import unittest
#import pyxb.binding.facets as facets
#import pyxb.binding.datatypes as datatypes
import pyxb.binding
import pyxb.binding.facets as facets
import pyxb.binding.datatypes as datatypes
from pyxb.utils import six
import sys

# 4.3.1 length
# 4.3.2 minLength
# 4.3.3 maxLength
# NEED 4.3.4 pattern
# 4.3.5 enumeration
# 4.3.6 whiteSpace
# 4.3.7 maxInclusive
# 4.3.8 maxExclusive
# 4.3.9 minExclusive
# 4.3.10 minInclusive
# 4.3.11 totalDigits
# 4.3.12 fractionDigits

class TLA (datatypes.string):
    pass
TLA._CF_length =  facets.CF_length(super_facet=datatypes.string._CF_length, value=datatypes.nonNegativeInteger(3))
TLA._InitializeFacetMap(TLA._CF_length)

class Password (datatypes.string):
    pass
Password._CF_minLength =  facets.CF_minLength(super_facet=datatypes.string._CF_minLength, value=datatypes.nonNegativeInteger(8))
Password._CF_maxLength =  facets.CF_maxLength(super_facet=datatypes.string._CF_maxLength, value=facets.CF_maxLength._ValueDatatype(15))
Password._InitializeFacetMap(Password._CF_minLength, Password._CF_maxLength)

class AFew (pyxb.binding.basis.STD_list):
    _ItemType = datatypes.integer
    pass
AFew._CF_minLength =  facets.CF_minLength(super_facet=datatypes.string._CF_minLength, value=datatypes.nonNegativeInteger(2))
AFew._CF_maxLength =  facets.CF_maxLength(super_facet=datatypes.string._CF_maxLength, value=facets.CF_maxLength._ValueDatatype(4))
AFew._InitializeFacetMap(AFew._CF_minLength, AFew._CF_maxLength)

class Hand (datatypes.NMTOKENS):
    pass
Hand._CF_length =  facets.CF_length(super_facet=datatypes.string._CF_length, value=datatypes.nonNegativeInteger(5))
Hand._InitializeFacetMap(Hand._CF_length)

class Cardinals (datatypes.string, facets._Enumeration_mixin):
    pass
Cardinals._CF_enumeration = facets.CF_enumeration(value_datatype=Cardinals, super_facet=datatypes.string._CF_enumeration, enum_prefix=None)
Cardinals.one = Cardinals._CF_enumeration.addEnumeration(unicode_value=six.u('one'))
Cardinals.two = Cardinals._CF_enumeration.addEnumeration(unicode_value=six.u('two'))
Cardinals.three = Cardinals._CF_enumeration.addEnumeration(unicode_value=six.u('three'))
Cardinals._InitializeFacetMap(Cardinals._CF_enumeration)

class ExclusiveFloat (datatypes.float):
    pass
ExclusiveFloat._CF_minExclusive =  facets.CF_minExclusive(super_facet=datatypes.float._CF_minExclusive, value_datatype=datatypes.float, value=datatypes.float(-5))
ExclusiveFloat._CF_maxExclusive =  facets.CF_maxExclusive(super_facet=datatypes.float._CF_maxExclusive, value_datatype=datatypes.float, value=datatypes.float(7))
ExclusiveFloat._InitializeFacetMap(ExclusiveFloat._CF_minExclusive, ExclusiveFloat._CF_maxExclusive)

class FixedPoint (datatypes.decimal):
    pass
FixedPoint._CF_totalDigits =  facets.CF_totalDigits(super_facet=datatypes.decimal._CF_totalDigits, value=facets.CF_totalDigits._ValueDatatype(5))
FixedPoint._CF_fractionDigits =  facets.CF_fractionDigits(super_facet=datatypes.decimal._CF_maxExclusive, value=facets.CF_fractionDigits._ValueDatatype(2))
FixedPoint._InitializeFacetMap(FixedPoint._CF_totalDigits, FixedPoint._CF_fractionDigits)

class ReplaceString (datatypes.string):
    pass
ReplaceString._CF_whiteSpace = facets.CF_whiteSpace(value=facets._WhiteSpace_enum.replace, super_facet=datatypes.string._CF_whiteSpace)
ReplaceString._InitializeFacetMap(ReplaceString._CF_whiteSpace)

class CollapseString (datatypes.string):
    pass
CollapseString._CF_whiteSpace = facets.CF_whiteSpace(value=facets._WhiteSpace_enum.collapse, super_facet=datatypes.string._CF_whiteSpace)
CollapseString._InitializeFacetMap(CollapseString._CF_whiteSpace)

class testMaxInclusive (unittest.TestCase):
    def test (self):
        self.assertEqual(5, datatypes.byte(5))
        self.assertRaises(SimpleTypeValueError, datatypes.byte, 256)
        self.assertRaises(SimpleTypeValueError, datatypes.byte, 255)
        self.assertEqual(127, datatypes.byte(127))
        self.assertRaises(SimpleTypeValueError, datatypes.byte, -150)
        self.assertRaises(SimpleTypeValueError, datatypes.byte, -129)
        self.assertEqual(-128, datatypes.byte(-128))

class testMinMaxExclusive (unittest.TestCase):
    def test (self):
        self.assertRaises(SimpleFacetValueError, ExclusiveFloat, -10.0)
        self.assertRaises(SimpleFacetValueError, ExclusiveFloat, -5.0)
        self.assertEqual(-4.0, ExclusiveFloat(-4.0))
        self.assertEqual(0.0, ExclusiveFloat(0.0))
        self.assertEqual(6.875, ExclusiveFloat(6.875))
        self.assertRaises(SimpleFacetValueError, ExclusiveFloat, 7.0)

class testLength (unittest.TestCase):
    def test (self):
        self.assertEqual('one', TLA('one'))
        self.assertRaises(SimpleFacetValueError, TLA, 'three')
        self.assertRaises(SimpleFacetValueError, TLA, 'un')

    def testList (self):
        t = datatypes.NMTOKEN('card')
        self.assertRaises(SimpleFacetValueError, Hand, 4 * [t])
        self.assertEqual(5, len(Hand(5 * [t])))
        self.assertRaises(SimpleFacetValueError, Hand, 6 * [t])

class testMinMaxLength (unittest.TestCase):
    def test (self):
        self.assertRaises(SimpleFacetValueError, Password, 7*'x')
        self.assertEqual(8 * 'x', Password(8*'x'))
        self.assertEqual(15 * 'x', Password(15*'x'))
        self.assertRaises(SimpleFacetValueError, Password, 16*'x')

    def testList (self):
        i = datatypes.integer(2)
        self.assertRaises(SimpleFacetValueError, AFew, [])
        self.assertRaises(SimpleFacetValueError, AFew, [i])
        self.assertEqual(2, len(AFew(2 * [i])))
        self.assertEqual(3, len(AFew(3 * [i])))
        self.assertEqual(4, len(AFew(4 * [i])))
        self.assertRaises(SimpleFacetValueError, AFew, 5 * [i])

class testEnumeration (unittest.TestCase):
    def test (self):
        self.assertEqual(Cardinals.one, Cardinals('one'))
        self.assertEqual(Cardinals.one, Cardinals(six.u('one')))
        self.assertEqual(Cardinals.three, Cardinals('three'))
        self.assertEqual(Cardinals.three, Cardinals(six.u('three')))
        self.assertRaises(SimpleFacetValueError, Cardinals, 'One')
        self.assertRaises(SimpleFacetValueError, Cardinals, 'four')
        if sys.version_info[:2] >= (2, 7):
            with self.assertRaises(SimpleFacetValueError) as cm:
                Cardinals('four')
            self.assertEqual(cm.exception.facet, Cardinals._CF_enumeration)

class testDigits (unittest.TestCase):
    # FixedPoint is 5.2 (-999.99 through 999.99 for fractional)
    def assertAlmostEqual (self, v1, v2, *args, **kw):
        if (isinstance(v1, datatypes.decimal)
            or isinstance(v2, datatypes.decimal)):
            if not isinstance(v1, datatypes.decimal):
                v1 = datatypes.decimal(str(v1))
            if not isinstance(v2, datatypes.decimal):
                v2 = datatypes.decimal(str(v2))
        return super(testDigits, self).assertAlmostEqual(v1, v2, *args, **kw)

    def testTotalDigits (self):
        self.assertRaises(SimpleFacetValueError, FixedPoint, '0.001')
        self.assertAlmostEqual(0.01, FixedPoint('0.01'))
        self.assertAlmostEqual(0.12, FixedPoint('0.12'))
        self.assertRaises(SimpleFacetValueError, FixedPoint, '12.345')
        self.assertAlmostEqual(123.45, FixedPoint('123.45'))
        self.assertAlmostEqual(1, FixedPoint('1'))
        self.assertAlmostEqual(12, FixedPoint('12'))
        self.assertAlmostEqual(123, FixedPoint('123'))
        self.assertAlmostEqual(1234, FixedPoint('1234'))
        self.assertAlmostEqual(12345, FixedPoint('12345'))
        self.assertRaises(SimpleFacetValueError, FixedPoint, '123000')
        self.assertRaises(SimpleFacetValueError, FixedPoint, '123456')
        self.assertRaises(SimpleFacetValueError, FixedPoint, '1234567')
        self.assertRaises(SimpleFacetValueError, FixedPoint, '-0.001')
        self.assertAlmostEqual(-0.01, FixedPoint('-0.01'))
        self.assertAlmostEqual(-0.12, FixedPoint('-0.12'))
        self.assertRaises(SimpleFacetValueError, FixedPoint, '-12.345')
        self.assertAlmostEqual(-123.45, FixedPoint('-123.45'))
        self.assertAlmostEqual(-1, FixedPoint('-1'))
        self.assertAlmostEqual(-12, FixedPoint('-12'))
        self.assertAlmostEqual(-123, FixedPoint('-123'))
        self.assertAlmostEqual(-1234, FixedPoint('-1234'))
        self.assertAlmostEqual(-12345, FixedPoint('-12345'))
        self.assertRaises(SimpleFacetValueError, FixedPoint, '-123000')
        self.assertRaises(SimpleFacetValueError, FixedPoint, '-123456')
        self.assertRaises(SimpleFacetValueError, FixedPoint, '-1234567')

    def testFractionDigits (self):
        self.assertAlmostEqual(1, FixedPoint('1'))
        self.assertAlmostEqual(1.2, FixedPoint('1.2'))
        self.assertAlmostEqual(1.23, FixedPoint('1.23'))
        self.assertRaises(SimpleFacetValueError, FixedPoint, '1.234')
        self.assertAlmostEqual(12, FixedPoint('12'))
        self.assertAlmostEqual(12.3, FixedPoint('12.3'))
        self.assertAlmostEqual(12.34, FixedPoint('12.34'))
        self.assertRaises(SimpleFacetValueError, FixedPoint, '12.345')
        self.assertAlmostEqual(-1, FixedPoint('-1'))
        self.assertAlmostEqual(-1.2, FixedPoint('-1.2'))
        self.assertAlmostEqual(-1.23, FixedPoint('-1.23'))
        self.assertRaises(SimpleFacetValueError, FixedPoint, '-1.234')
        self.assertAlmostEqual(-12, FixedPoint('-12'))
        self.assertAlmostEqual(-12.3, FixedPoint('-12.3'))
        self.assertAlmostEqual(-12.34, FixedPoint('-12.34'))
        self.assertRaises(SimpleFacetValueError, FixedPoint, '-12.345')

    def testLexical (self):
        self.assertEqual('1.0', FixedPoint('1').xsdLiteral())
        self.assertEqual('0.1', FixedPoint('0.1').xsdLiteral())
        self.assertEqual('1.0', FixedPoint('1.0000').xsdLiteral())
        self.assertEqual('0.1', FixedPoint('0000.10000').xsdLiteral())
        self.assertEqual('1.0', FixedPoint('+1').xsdLiteral())
        self.assertEqual('-1.0', FixedPoint('-1').xsdLiteral())

class testWhitespace (unittest.TestCase):
    __Preserve = facets.CF_whiteSpace(value=facets._WhiteSpace_enum.preserve)
    def testPreserve (self):
        cases = [ "test", "  test", "  test", "\ttest", "\ttest\n\rtoo\n" ]
        for c in cases:
            self.assertEqual(c, self.__Preserve.normalizeString(c))

    def testPreserveString (self):
        cases = [ "test", "  test", "  test", "\ttest", "\ttest\n\rtoo\n" ]
        for c in cases:
            self.assertEqual(c, datatypes.string(c))
            self.assertEqual(c, datatypes.string(six.text_type(c)))

    __Replace = facets.CF_whiteSpace(value=facets._WhiteSpace_enum.replace)
    def testReplace (self):
        self.assertEqual("test", self.__Replace.normalizeString("test"))
        self.assertEqual("  test", self.__Replace.normalizeString("  test"))
        self.assertEqual(" test", self.__Replace.normalizeString("\ttest"))
        self.assertEqual(" test  ", self.__Replace.normalizeString("\ttest\n\r"))
        self.assertEqual(" test  too ", self.__Replace.normalizeString("\ttest\n\rtoo\n"))
        self.assertEqual("test", self.__Replace.normalizeString(six.u("test")))
        self.assertEqual("  test", self.__Replace.normalizeString(six.u("  test")))
        self.assertEqual(" test", self.__Replace.normalizeString(six.u("\ttest")))
        self.assertEqual(" test  ", self.__Replace.normalizeString(six.u("\ttest\n\r")))
        self.assertEqual(" test  too ", self.__Replace.normalizeString(six.u("\ttest\n\rtoo\n")))

    def testReplaceString (self):
        self.assertEqual("test", ReplaceString("test", _from_xml=True))
        self.assertEqual("  test", ReplaceString("  test", _from_xml=True))
        self.assertEqual(" test", ReplaceString("\ttest", _from_xml=True))
        self.assertEqual(" test  ", ReplaceString("\ttest\n\r", _from_xml=True))
        self.assertEqual(" test  too ", ReplaceString("\ttest\n\rtoo\n", _from_xml=True))
        self.assertEqual("test", ReplaceString(six.u("test"), _from_xml=True))
        self.assertEqual("  test", ReplaceString(six.u("  test"), _from_xml=True))
        self.assertEqual(" test", ReplaceString(six.u("\ttest"), _from_xml=True))
        self.assertEqual(" test  ", ReplaceString(six.u("\ttest\n\r"), _from_xml=True))
        self.assertEqual(" test  too ", ReplaceString(six.u("\ttest\n\rtoo\n"), _from_xml=True))

    __Collapse = facets.CF_whiteSpace(value=facets._WhiteSpace_enum.collapse)
    def testCollapse (self):
        self.assertEqual("test", self.__Collapse.normalizeString("test"))
        self.assertEqual("test", self.__Collapse.normalizeString("  test"))
        self.assertEqual("test", self.__Collapse.normalizeString("\ttest"))
        self.assertEqual("test", self.__Collapse.normalizeString("\ttest\n\r"))
        self.assertEqual("test too", self.__Collapse.normalizeString("\ttest\n\rtoo\n"))
        self.assertEqual("test", self.__Collapse.normalizeString(six.u("test")))
        self.assertEqual("test", self.__Collapse.normalizeString(six.u("  test")))
        self.assertEqual("test", self.__Collapse.normalizeString(six.u("\ttest")))
        self.assertEqual("test", self.__Collapse.normalizeString(six.u("\ttest\n\r")))
        self.assertEqual("test too", self.__Collapse.normalizeString(six.u("\ttest\n\rtoo\n")))

    def testCollapseString (self):
        # The way it will work if from DOM content:
        self.assertEqual("test", CollapseString("test", _from_xml=True))
        self.assertEqual("test", CollapseString("  test", _from_xml=True))
        self.assertEqual("test", CollapseString("\ttest", _from_xml=True))
        self.assertEqual("test", CollapseString("\ttest\n\r", _from_xml=True))
        self.assertEqual("test too", CollapseString("\ttest\n\rtoo\n", _from_xml=True))
        self.assertEqual("test", CollapseString(six.u("test"), _from_xml=True))
        self.assertEqual("test", CollapseString(six.u("  test"), _from_xml=True))
        self.assertEqual("test", CollapseString(six.u("\ttest"), _from_xml=True))
        self.assertEqual("test", CollapseString(six.u("\ttest\n\r"), _from_xml=True))
        self.assertEqual("test too", CollapseString(six.u("\ttest\n\rtoo\n"), _from_xml=True))

    def testApplyWhitespace (self):
        goal = 'one two'
        source = '  one    two   '
        self.assertEqual(goal, CollapseString(goal, _from_xml=True))
        self.assertEqual(goal, CollapseString(source, _from_xml=True))
        self.assertEqual(source, CollapseString(source, _apply_whitespace_facet=False, _from_xml=True))

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
# This file contains code adopted from http://code.activestate.com/recipes/.
# Copyright and licensing information is associated with each incorporated
# recipe.  At this time, all code is licensed under PSF.

# http://code.activestate.com/recipes/363841/
# by Lars Tiede, derivative from:
# http://code.activestate.com/recipes/52257/
# by Paul Prescod
# Licensed under PSF
# No changes
def detectXMLEncoding(fp):
    """ Attempts to detect the character encoding of the xml file
    given by a file object fp. fp must not be a codec wrapped file
    object!

    The return value can be:
        - if detection of the BOM succeeds, the codec name of the
        corresponding unicode charset is returned

        - if BOM detection fails, the xml declaration is searched for
        the encoding attribute and its value returned. the "<"
        character has to be the very first in the file then (it's xml
        standard after all).

        - if BOM and xml declaration fail, None is returned. According
        to xml 1.0 it should be utf_8 then, but it wasn't detected by
        the means offered here. at least one can be pretty sure that a
        character coding including most of ASCII is used :-/
    """
    ### detection using BOM

    ## the BOMs we know, by their pattern
    bomDict={ # bytepattern : name
             (0x00, 0x00, 0xFE, 0xFF) : "utf_32_be",
             (0xFF, 0xFE, 0x00, 0x00) : "utf_32_le",
             (0xFE, 0xFF, None, None) : "utf_16_be",
             (0xFF, 0xFE, None, None) : "utf_16_le",
             (0xEF, 0xBB, 0xBF, None) : "utf_8",
            }

    ## go to beginning of file and get the first 4 bytes
    oldFP = fp.tell()
    fp.seek(0)
    (byte1, byte2, byte3, byte4) = tuple(map(ord, fp.read(4)))

    ## try bom detection using 4 bytes, 3 bytes, or 2 bytes
    bomDetection = bomDict.get((byte1, byte2, byte3, byte4))
    if not bomDetection :
        bomDetection = bomDict.get((byte1, byte2, byte3, None))
        if not bomDetection :
            bomDetection = bomDict.get((byte1, byte2, None, None))

    ## if BOM detected, we're done :-)
    if bomDetection :
        fp.seek(oldFP)
        return bomDetection


    ## still here? BOM detection failed.
    ##  now that BOM detection has failed we assume one byte character
    ##  encoding behaving ASCII - of course one could think of nice
    ##  algorithms further investigating on that matter, but I won't for now.


    ### search xml declaration for encoding attribute
    import re

    ## assume xml declaration fits into the first 2 KB (*cough*)
    fp.seek(0)
    buffer = fp.read(2048)

    ## set up regular expression
    xmlDeclPattern = r"""
    ^<\?xml             # w/o BOM, xmldecl starts with <?xml at the first byte
    .+?                 # some chars (version info), matched minimal
    encoding=           # encoding attribute begins
    ["']                # attribute start delimiter
    (?P<encstr>         # what's matched in the brackets will be named encstr
     [^"']+              # every character not delimiter (not overly exact!)
    )                   # closes the brackets pair for the named group
    ["']                # attribute end delimiter
    .*?                 # some chars optionally (standalone decl or whitespace)
    \?>                 # xmldecl end
    """

    xmlDeclRE = re.compile(xmlDeclPattern, re.VERBOSE)

    ## search and extract encoding string
    match = xmlDeclRE.search(buffer)
    fp.seek(oldFP)
    if match :
        return match.group("encstr")
    else :
        return None



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
from xml.dom import Node

import os.path
schema_path = os.path.abspath(os.path.join(os.path.dirname(__file__), '../schemas/test-deconflict.xsd'))
code = pyxb.binding.generate.GeneratePython(schema_location=schema_path)

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

class TestDeconflict (unittest.TestCase):
    def testAttributes (self):
        self.assertEqual(2, len(CTD_empty._ElementMap))
        ef = CTD_empty._ElementMap['content']
        self.assertEqual('content_', ef.id())
        self.assertFalse(ef.isPlural())
        self.assertTrue(ef.defaultValue() is None)
        ef = CTD_empty._ElementMap['Factory']
        self.assertEqual('Factory_', ef.id())
        self.assertFalse(ef.isPlural())
        self.assertTrue(ef.defaultValue() is None)
        self.assertEqual(2, len(CTD_empty._AttributeMap))
        self.assertEqual('toDOM_', CTD_empty._AttributeMap['toDOM'].id())
        self.assertEqual('Factory_2', CTD_empty._AttributeMap['Factory'].id())

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import unittest
from pyxb.utils.utility import *
from pyxb.utils.utility import _DeconflictSymbols_mixin
from pyxb.utils import six
import sys

class DST_base (_DeconflictSymbols_mixin):
    _ReservedSymbols = set([ 'one', 'two' ])

class DST_sub (DST_base):
    _ReservedSymbols = DST_base._ReservedSymbols.union(set([ 'three' ]))

class DeconfictSymbolsTtest (unittest.TestCase):
    def testDeconflict (self):
        self.assertEqual(2, len(DST_base._ReservedSymbols))
        self.assertEqual(3, len(DST_sub._ReservedSymbols))

if six.PY3:
    def qu(s):
        return '"' + s.replace('"', '\\"') + '"'
else:
    def qu(s):
        s = unicode(s.replace(r'\\', r'\\\\'), "unicode_escape")
        return six.u('u"') + s.replace('"', '\\"') + '"'

class BasicTest (unittest.TestCase):

    cases = ( ( r'"1\x042"', "1\0042" ) # expanded octal
            , ( r'"1\x042"', '1\0042' ) # expanded octal (single quotes do not affect escaping)
            , ( "r'1\\0042'", r'1\0042' ) # preserve unexpanded octal
            , ( r'"1\x222&3"', '1"2&3' )  # escape double quotes
            , ( '"one\'two"', "one'two" ) # preserve single quote
            , ( r'"1\n2"', "1\n2" )       # expanded newline to escape sequence
            , ( "r'1\\n2'", r'1\n2' )     # raw backslash preserved
            , ( "\"1'\\n'2\"", "1'\n'2" ) # expanded newline to escape sequence
            , ( "\"1'\\n'2\"", '1\'\n\'2' ) # expanded newline to escape sequence (single quotes)
            , ( "\"1\\x22\\n\\x222\"", '1"\n"2' ) # escape double quotes around expanded newline
            , ( "r'1\\'\\n\\'2'", r'1\'\n\'2' )   # preserve escaped quote and newline
            , ( qu(r'"1\u00042"'), six.u("1\0042") )       # unicode expanded octal
            , ( qu(r'"1\u00222&3"'), six.u('1"2&3') )      # unicode escape double quotes
            , ( qu(r'"one' + "'" + r'two"'), six.u("one'two") ) # unicode embedded single quote
            , ( "r'\\i\\c*'", r'\i\c*' )               # backslashes as in patterns
            , ( qu('"0"'), six.u('\u0030') )                   # expanded unicode works
            , ( qu('"\\u0022"'), six.u('"') )      # unicode double quotes are escaped
            , ( qu('"\\u0022"'), six.u('\u0022') ) # single quotes don't change that expanded unicode works
            , ( qu('"\\u0022"'), six.u(r'\u0022') ) # raw has no effect on unicode escapes
            , ( qu('\"'), six.u("'") )           # unicode single quote works
            , ( qu('\"\\u00220\\u0022\"'), six.u('"\u0030"') ) # unicode with double quotes works
            )


    def testPrepareIdentifier (self):
        in_use = set()
        self.assertEqual('id', PrepareIdentifier('id', in_use))
        self.assertEqual('id_', PrepareIdentifier('id', in_use))
        self.assertEqual('id_2', PrepareIdentifier('id_', in_use))
        self.assertEqual('id_3', PrepareIdentifier('id____', in_use))
        self.assertEqual('_id', PrepareIdentifier('id', in_use, protected=True))
        self.assertEqual('_id_', PrepareIdentifier('id', in_use, protected=True))
        self.assertEqual('__id', PrepareIdentifier('id', in_use, private=True))
        self.assertEqual('__id_', PrepareIdentifier('id', in_use, private=True))

        reserved = frozenset([ 'Factory' ])
        in_use = set()
        self.assertEqual('Factory_', PrepareIdentifier('Factory', in_use, reserved))
        self.assertEqual('Factory_2', PrepareIdentifier('Factory', in_use, reserved))
        self.assertEqual('Factory_3', PrepareIdentifier('Factory', in_use, reserved))

        in_use = set()
        self.assertEqual('global_', PrepareIdentifier('global', in_use))
        self.assertEqual('global_2', PrepareIdentifier('global', in_use))
        self.assertEqual('global_3', PrepareIdentifier('global', in_use))

        in_use = set()
        self.assertEqual('n24_hours', PrepareIdentifier('24 hours', in_use))

    def testQuotedEscape (self):
        for ( expected, input ) in self.cases:
            result = QuotedEscaped(input)
            # Given "expected" value may not be correct.  Don't care as
            # long as the evalution produces the input.
            #self.assertEqual(expected, result)
            self.assertEqual(input, eval(result))

    def testMakeIdentifier (self):
        self.assertEqual('id', MakeIdentifier('id'))
        self.assertEqual('id', MakeIdentifier(six.u('id')))
        self.assertEqual('id_sep', MakeIdentifier(six.u('id_sep')))
        self.assertEqual('id_sep', MakeIdentifier(six.u('id sep')))
        self.assertEqual('id_sep_too', MakeIdentifier(six.u('id-sep too')))
        self.assertEqual('idid', MakeIdentifier(six.u('id&id')))
        self.assertEqual('id', MakeIdentifier('_id'))
        self.assertEqual('id_', MakeIdentifier('_id_'))
        self.assertEqual('emptyString', MakeIdentifier(''))
        self.assertEqual('emptyString', MakeIdentifier('_'))

    def testCamelCase (self):
        self.assertEqual('one_and_two', MakeIdentifier('one_and_two'))
        self.assertEqual('one_and_two', MakeIdentifier('one-and-two'))
        self.assertEqual('oneAndTwo', MakeIdentifier('one_and_two', camel_case=True))
        self.assertEqual('oneAndTwo', MakeIdentifier('one-and-two', camel_case=True))

    def testDeconflictKeyword (self):
        self.assertEqual('id', DeconflictKeyword('id'))
        self.assertEqual('for_', DeconflictKeyword('for'))

    def testMakeUnique (self):
        in_use = set()
        self.assertEqual('id', MakeUnique('id', in_use))
        self.assertEqual(1, len(in_use))
        self.assertEqual('id_', MakeUnique('id', in_use))
        self.assertEqual(2, len(in_use))
        self.assertEqual('id_2', MakeUnique('id', in_use))
        self.assertEqual(3, len(in_use))
        self.assertEqual(set(( 'id', 'id_', 'id_2' )), in_use)

class TestGraph (unittest.TestCase):

    _Edges = [
        (1, 2),
        (2, 3),
        (2, 4),
        (4, 8),
        (5, 6),
        (5, 7),
        (6, 10),
        (7, 8),
        (8, 5),
        (8, 9),
        (9, 10),
        (10, 0)
        ]

    def testRoot (self):
        graph = Graph()
        [ graph.addEdge(*_e) for _e in self._Edges ]
        roots = graph.roots().copy()
        self.assertEqual(1, len(roots))
        self.assertEqual(1, roots.pop())

    def testTarjan (self):
        graph = Graph()
        [ graph.addEdge(*_e) for _e in self._Edges ]
        scc = graph.scc()
        self.assertEqual(1, len(scc))
        self.assertEqual(set([5, 7, 8]), set(scc[0]))

    def testRootIsNode (self):
        graph = Graph()
        graph.addRoot(4)
        self.assertEqual(1, len(graph.nodes()))
        self.assertEqual(graph.roots(), graph.nodes())

    def testDFSOrder1 (self):
        graph = Graph()
        [ graph.addEdge(*_e) for _e in self._Edges ]
        order = graph.dfsOrder()
        self.assertTrue(isinstance(order, list))
        self.assertEqual(len(graph.nodes()), len(order))
        walked = set()
        for source in order:
            for target in graph.edgeMap().get(source, []):
                self.assertTrue((target in walked) or (graph.sccForNode(source) == graph.sccForNode(target)), '%s -> %s not satisfied, seen' % (source, target))
            walked.add(source)
        order = graph.sccOrder()
        self.assertEqual(len(graph.nodes()), len(order) + 2)
        walked = set()
        for source in order:
            if isinstance(source, list):
                walked.update(source)
                continue
            for target in graph.edgeMap().get(source, []):
                self.assertTrue((target in walked) or (graph.sccForNode(source) == graph.sccForNode(target)), '%s -> %s not satisfied, seen' % (source, target))
            walked.add(source)

    def testDFSOrder2 (self):
        graph = Graph()
        graph.addEdge(2, 2)
        graph.addEdge(2, 1)
        graph.addNode(3)
        order = graph.dfsOrder()
        self.assertTrue(isinstance(order, list))
        self.assertEqual(len(graph.nodes()), len(order))
        walked = set()
        for source in order:
            for target in graph.edgeMap().get(source, []):
                self.assertTrue((target in walked) or (graph.sccForNode(source) == graph.sccForNode(target)), '%s -> %s not satisfied, seen' % (source, target))
            walked.add(source)

    def testDFSOrder_Loop (self):
        graph = Graph()
        graph.addEdge(1, 2)
        graph.addEdge(2, 3)
        graph.addEdge(3, 1)
        self.assertEqual(0, len(graph.roots()))
        self.assertRaises(Exception, graph.dfsOrder)
        graph.addRoot(1)
        self.assertEqual(1, len(graph.roots()))
        scc = graph.scc()
        self.assertEqual(1, len(scc))
        self.assertEqual(set([1, 2, 3]), set(scc[0]))

    def testDFSOrder4 (self):
        graph = Graph()
        graph.addEdge('gmd.applicationSchema', 'gco.basicTypes')
        graph.addEdge('gmd.applicationSchema', 'gmd.applicationSchema')
        graph.addEdge('gmd.applicationSchema', 'xlink.xlinks')
        graph.addEdge('gmd.applicationSchema', 'gco.gcoBase')
        graph.addEdge('gmd.applicationSchema', 'gmd.citation')
        graph.addEdge('gmd.applicationSchema', 'gml.basicTypes')
        graph.addEdge('gmd.portrayalCatalogue', 'gco.gcoBase')
        graph.addEdge('gmd.portrayalCatalogue', 'gmd.portrayalCatalogue')
        graph.addEdge('gmd.portrayalCatalogue', 'gmd.citation')
        graph.addEdge('gmd.portrayalCatalogue', 'xlink.xlinks')
        graph.addEdge('gmd.portrayalCatalogue', 'gml.basicTypes')
        graph.addEdge('gmd.freeText', 'gmd.freeText')
        graph.addEdge('gmd.freeText', 'gco.gcoBase')
        graph.addEdge('gmd.freeText', 'gco.basicTypes')
        graph.addEdge('gmd.freeText', 'gmd.citation')
        graph.addEdge('gmd.freeText', 'gmd.identification')
        graph.addEdge('gmd.freeText', 'gml.basicTypes')
        graph.addEdge('gmd.freeText', 'xlink.xlinks')
        graph.addEdge('gmd.dataQuality', 'gco.basicTypes')
        graph.addEdge('gmd.dataQuality', 'xlink.xlinks')
        graph.addEdge('gmd.dataQuality', 'gco.gcoBase')
        graph.addEdge('gmd.dataQuality', 'gmd.referenceSystem')
        graph.addEdge('gmd.dataQuality', 'gmd.maintenance')
        graph.addEdge('gmd.dataQuality', 'gmd.extent')
        graph.addEdge('gmd.dataQuality', 'gmd.citation')
        graph.addEdge('gmd.dataQuality', 'gmd.identification')
        graph.addEdge('gmd.dataQuality', 'gml.basicTypes')
        graph.addEdge('gmd.dataQuality', 'gmd.dataQuality')
        graph.addEdge('gmd.metadataApplication', 'gml.basicTypes')
        graph.addEdge('gmd.metadataApplication', 'gco.gcoBase')
        graph.addEdge('gmd.metadataApplication', 'gmd.metadataEntity')
        graph.addEdge('gmd.metadataApplication', 'gmd.metadataApplication')
        graph.addEdge('gmd.metadataApplication', 'xlink.xlinks')
        graph.addEdge('gmd.spatialRepresentation', 'gco.gcoBase')
        graph.addEdge('gmd.spatialRepresentation', 'gss.geometry')
        graph.addEdge('gmd.spatialRepresentation', 'gml.basicTypes')
        graph.addEdge('gmd.spatialRepresentation', 'gco.basicTypes')
        graph.addEdge('gmd.spatialRepresentation', 'gmd.citation')
        graph.addEdge('gmd.spatialRepresentation', 'xlink.xlinks')
        graph.addEdge('gmd.spatialRepresentation', 'gmd.spatialRepresentation')
        graph.addEdge('gmd.extent', 'gco.basicTypes')
        graph.addEdge('gmd.extent', 'gts.temporalObjects')
        graph.addEdge('gmd.extent', 'gco.gcoBase')
        graph.addEdge('gmd.extent', 'gmd.extent')
        graph.addEdge('gmd.extent', 'gml.basicTypes')
        graph.addEdge('gmd.extent', 'gmd.referenceSystem')
        graph.addEdge('gmd.extent', 'xlink.xlinks')
        graph.addEdge('gmd.extent', 'gss.geometry')
        graph.addEdge('gmd.extent', 'gsr.spatialReferencing')
        graph.addEdge('gmd.distribution', 'gmd.distribution')
        graph.addEdge('gmd.distribution', 'gml.basicTypes')
        graph.addEdge('gmd.distribution', 'gco.gcoBase')
        graph.addEdge('gmd.distribution', 'gco.basicTypes')
        graph.addEdge('gmd.distribution', 'gmd.citation')
        graph.addEdge('gmd.distribution', 'xlink.xlinks')
        graph.addEdge('gmd.metadataExtension', 'gml.basicTypes')
        graph.addEdge('gmd.metadataExtension', 'gco.gcoBase')
        graph.addEdge('gmd.metadataExtension', 'gco.basicTypes')
        graph.addEdge('gmd.metadataExtension', 'gmd.metadataExtension')
        graph.addEdge('gmd.metadataExtension', 'gmd.citation')
        graph.addEdge('gmd.metadataExtension', 'xlink.xlinks')
        graph.addEdge('gmd.maintenance', 'gts.temporalObjects')
        graph.addEdge('gmd.maintenance', 'gco.gcoBase')
        graph.addEdge('gmd.maintenance', 'gmd.maintenance')
        graph.addEdge('gmd.maintenance', 'gco.basicTypes')
        graph.addEdge('gmd.maintenance', 'gmd.citation')
        graph.addEdge('gmd.maintenance', 'gml.basicTypes')
        graph.addEdge('gmd.maintenance', 'xlink.xlinks')
        graph.addEdge('gmd.identification', 'gco.basicTypes')
        graph.addEdge('gmd.identification', 'xlink.xlinks')
        graph.addEdge('gmd.identification', 'gco.gcoBase')
        graph.addEdge('gmd.identification', 'gmd.referenceSystem')
        graph.addEdge('gmd.identification', 'gmd.maintenance')
        graph.addEdge('gmd.identification', 'gmd.extent')
        graph.addEdge('gmd.identification', 'gmd.distribution')
        graph.addEdge('gmd.identification', 'gmd.citation')
        graph.addEdge('gmd.identification', 'gmd.identification')
        graph.addEdge('gmd.identification', 'gml.basicTypes')
        graph.addEdge('gmd.identification', 'gmd.constraints')
        graph.addEdge('gmd.metadataEntity', 'gmd.content')
        graph.addEdge('gmd.metadataEntity', 'gco.basicTypes')
        graph.addEdge('gmd.metadataEntity', 'gmd.portrayalCatalogue')
        graph.addEdge('gmd.metadataEntity', 'gmd.metadataApplication')
        graph.addEdge('gmd.metadataEntity', 'gco.gcoBase')
        graph.addEdge('gmd.metadataEntity', 'gml.basicTypes')
        graph.addEdge('gmd.metadataEntity', 'gmd.applicationSchema')
        graph.addEdge('gmd.metadataEntity', 'gmd.metadataEntity')
        graph.addEdge('gmd.metadataEntity', 'gmd.referenceSystem')
        graph.addEdge('gmd.metadataEntity', 'gmd.maintenance')
        graph.addEdge('gmd.metadataEntity', 'gmd.metadataExtension')
        graph.addEdge('gmd.metadataEntity', 'gmd.distribution')
        graph.addEdge('gmd.metadataEntity', 'gmd.freeText')
        graph.addEdge('gmd.metadataEntity', 'gmd.identification')
        graph.addEdge('gmd.metadataEntity', 'gmd.constraints')
        graph.addEdge('gmd.metadataEntity', 'xlink.xlinks')
        graph.addEdge('gmd.metadataEntity', 'gmd.citation')
        graph.addEdge('gmd.metadataEntity', 'gmd.dataQuality')
        graph.addEdge('gmd.metadataEntity', 'gmd.spatialRepresentation')
        graph.addEdge('gmd.constraints', 'gco.gcoBase')
        graph.addEdge('gmd.constraints', 'gco.basicTypes')
        graph.addEdge('gmd.constraints', 'gmd.constraints')
        graph.addEdge('gmd.constraints', 'xlink.xlinks')
        graph.addEdge('gmd.constraints', 'gml.basicTypes')
        graph.addEdge('gmd.content', 'gmd.content')
        graph.addEdge('gmd.content', 'gco.basicTypes')
        graph.addEdge('gmd.content', 'xlink.xlinks')
        graph.addEdge('gmd.content', 'gco.gcoBase')
        graph.addEdge('gmd.content', 'gmd.referenceSystem')
        graph.addEdge('gmd.content', 'gmd.citation')
        graph.addEdge('gmd.content', 'gml.basicTypes')
        graph.addEdge('gmd.referenceSystem', 'gco.gcoBase')
        graph.addEdge('gmd.referenceSystem', 'gco.basicTypes')
        graph.addEdge('gmd.referenceSystem', 'gmd.referenceSystem')
        graph.addEdge('gmd.referenceSystem', 'gmd.extent')
        graph.addEdge('gmd.referenceSystem', 'gmd.citation')
        graph.addEdge('gmd.referenceSystem', 'gml.basicTypes')
        graph.addEdge('gmd.referenceSystem', 'xlink.xlinks')
        graph.addEdge('gmd.citation', 'gco.gcoBase')
        graph.addEdge('gmd.citation', 'gml.basicTypes')
        graph.addEdge('gmd.citation', 'gco.basicTypes')
        graph.addEdge('gmd.citation', 'gmd.referenceSystem')
        graph.addEdge('gmd.citation', 'gmd.citation')
        graph.addEdge('gmd.citation', 'xlink.xlinks')

        self.assertEqual(23, len(graph.nodes()))
        self.assertEqual(123, len(graph.edges()))
        self.assertRaises(Exception, graph.dfsOrder)

    def testDFSOrder5 (self):
        graph = Graph()
        graph.addEdge(1, 2)
        graph.addEdge(1, 3)
        graph.addEdge(3, 4)
        graph.addEdge(3, 5)
        graph.addEdge(5, 1)
        graph.addEdge(5, 6)
        graph.addEdge(6, 2)
        self.assertRaises(Exception, graph.scc)
        self.assertRaises(Exception, graph.dfsOrder)
        graph.addRoot(1)
        order = graph.sccOrder(reset=True)
        self.assertEqual(4, len(order))
        self.assertEqual(1, len(graph.scc()))
        self.assertEqual(set([1, 3, 5]), set(graph.scc()[0]))

import tempfile

class _TestOpenOrCreate_mixin (object):
    def setUp (self):
        tf = tempfile.NamedTemporaryFile()
        self.__fileName = tf.name
        tf.close()

    def fileName (self):
        return self.__fileName

    def unlinkFile (self):
        try:
            os.unlink(self.__fileName)
        except OSError as e:
            if errno.ENOENT != e:
                raise

    def tearDown (self):
        self.unlinkFile()

class TestOpenOrCreate_New (unittest.TestCase, _TestOpenOrCreate_mixin):
    setUp = _TestOpenOrCreate_mixin.setUp
    tearDown = _TestOpenOrCreate_mixin.tearDown

    def testNew (self):
        filename = self.fileName()
        of = OpenOrCreate(filename)
        text = 'hello'
        texd = text.encode('utf-8')
        of.write(texd)
        of.close()

class TestOpenOrCreate_Local (unittest.TestCase, _TestOpenOrCreate_mixin):
    setUp = _TestOpenOrCreate_mixin.setUp
    tearDown = _TestOpenOrCreate_mixin.tearDown

    def testNew (self):
        filename = self.fileName()
        (path, localname) = os.path.split(filename)
        cwd = os.getcwd()
        try:
            os.chdir(path)
            of = OpenOrCreate(localname)
        finally:
            os.chdir(cwd)
        text = 'hello'
        texd = text.encode('utf-8')
        of.write(texd)
        of.close()

class TestOpenOrCreate_ExistingTagMatch (unittest.TestCase, _TestOpenOrCreate_mixin):
    setUp = _TestOpenOrCreate_mixin.setUp
    tearDown = _TestOpenOrCreate_mixin.tearDown

    def testExistingTagMatch (self):
        filename = self.fileName()
        tag = 'MyTagXX'
        text = 'This file has the tag %s in it' % (tag,)
        texd = text.encode('utf-8')
        fd = open(filename, 'wb')
        fd.write(texd)
        fd.close()
        fd = open(filename, 'rb')
        self.assertEqual(texd, fd.read())
        fd.close()
        of = OpenOrCreate(filename, tag=tag)
        text = 'New version with tag %s' % (tag,)
        texd = text.encode('utf-8')
        of.write(texd)
        of.close()
        fd = open(filename, 'rb')
        self.assertEqual(texd, fd.read())

class TestOpenOrCreate_ExistingTagMismatch (unittest.TestCase, _TestOpenOrCreate_mixin):
    setUp = _TestOpenOrCreate_mixin.setUp
    tearDown = _TestOpenOrCreate_mixin.tearDown

    def testExistingTagMismatch (self):
        filename = self.fileName()
        tag = 'MyTagXX'
        text = 'This file has the tag NotMyTag in it'
        texd = text.encode('utf-8')
        fd = open(filename, 'wb')
        fd.write(texd)
        fd.close()
        # Verify that opening for append will be positioned after the text.
        if 'win32' != sys.platform:
            fd = open(filename, 'a')
            self.assertTrue(0 < fd.tell())
            fd.close()
        fd = open(filename, 'rb')
        self.assertEqual(texd, fd.read())
        fd.close()
        if sys.version_info[:2] < (2, 7):
            self.assertRaises(OSError, OpenOrCreate, filename, tag=tag)
        else:
            with self.assertRaises(OSError) as cm:
                OpenOrCreate(filename, tag=tag)
            e = cm.exception
            self.assertEqual(e.errno, errno.EEXIST)

class TestHashForText (unittest.TestCase):

    def testBasic (self):
        text = 'This is some text'
        self.assertEqual('482cb0cfcbed6740a2bcb659c9ccc22a4d27b369', HashForText(text))

import datetime
class TestUTCTimeZone (unittest.TestCase):

    def testConstructors (self):
        dt = datetime.datetime.now()
        utc = UTCOffsetTimeZone()
        self.assertEqual(datetime.timedelta(0), utc.utcoffset(dt))
        self.assertEqual('Z', utc.tzname(dt))

        utc = UTCOffsetTimeZone('Z')
        self.assertEqual(datetime.timedelta(0), utc.utcoffset(dt))
        self.assertEqual('Z', utc.tzname(dt))

        utc = UTCOffsetTimeZone('+00:00')
        self.assertEqual(datetime.timedelta(0), utc.utcoffset(dt))
        self.assertEqual('Z', utc.tzname(dt))

        utc = UTCOffsetTimeZone('-00:00')
        self.assertEqual(datetime.timedelta(0), utc.utcoffset(dt))
        self.assertEqual('Z', utc.tzname(dt))

        utc = UTCOffsetTimeZone(95)
        self.assertEqual(datetime.timedelta(minutes=95), utc.utcoffset(dt))
        self.assertEqual('+01:35', utc.tzname(dt))

        td = datetime.timedelta(hours=3, minutes=42)
        utc = UTCOffsetTimeZone(td)
        self.assertEqual(td, utc.utcoffset(dt))
        self.assertEqual('+03:42', utc.tzname(dt))

    def testRangeValidation (self):
        dt = datetime.datetime.now()
        utc = UTCOffsetTimeZone('+13:59')
        self.assertEqual(datetime.timedelta(hours=13, minutes=59), utc.utcoffset(dt))
        self.assertEqual('+13:59', utc.tzname(dt))

        utc = UTCOffsetTimeZone('+14:00')
        self.assertEqual(datetime.timedelta(hours=14), utc.utcoffset(dt))
        self.assertEqual('+14:00', utc.tzname(dt))

        utc = UTCOffsetTimeZone('-13:59')
        self.assertEqual(-datetime.timedelta(hours=13, minutes=59), utc.utcoffset(dt))
        self.assertEqual('-13:59', utc.tzname(dt))

        utc = UTCOffsetTimeZone(-14*60)
        self.assertEqual(datetime.timedelta(hours=-14), utc.utcoffset(dt))
        self.assertEqual('-14:00', utc.tzname(dt))

        self.assertRaises(ValueError, UTCOffsetTimeZone, '14:01')
        self.assertRaises(ValueError, UTCOffsetTimeZone, -14*60 - 1)

    def testComparison (self):
        utc_a = UTCOffsetTimeZone()
        utc_b = UTCOffsetTimeZone('+00:00')
        utc_c = UTCOffsetTimeZone('-00:00')
        self.assertNotEqual(id(utc_a), id(utc_b))
        self.assertEqual(utc_a, utc_b)
        self.assertEqual(utc_a, utc_c)
        self.assertEqual(utc_b, utc_c)
        utc_p1 = UTCOffsetTimeZone(60)
        utc_m1 = UTCOffsetTimeZone(-60)
        self.assertTrue(utc_a < utc_p1)
        self.assertTrue(utc_m1 < utc_a)

class TestLocalTimeZone (unittest.TestCase):
    pass

class TestUniqueIdentifier (unittest.TestCase):

    def testBasic (self):
        u1 = UniqueIdentifier('one')
        self.assertEqual('one', u1.uid())
        u1b = UniqueIdentifier('one')
        self.assertEqual(u1, u1b)
        self.assertEqual(id(u1), id(u1b))

    def testNoUID (self):
        u1 = UniqueIdentifier()
        self.assertTrue(u1.uid() is not None)
        u2 = UniqueIdentifier(u1)
        self.assertEqual(id(u1), id(u2))

    def testPickling (self):
        import pickle
        import io

        u1 = UniqueIdentifier()
        outdata = io.BytesIO()
        pickler = pickle.Pickler(outdata, -1)
        pickler.dump(u1)
        instr = io.BytesIO(outdata.getvalue())
        unpickler = pickle.Unpickler(instr)
        u2 = unpickler.load()
        self.assertEqual(u1.uid(), u2.uid())
        self.assertEqual(u1, u2)
        self.assertEqual(id(u1), id(u2))

    def testStringEquivalence (self):
        u1 = UniqueIdentifier()
        self.assertEqual(u1, u1.uid())

    def testHash (self):
        u1 = UniqueIdentifier()
        mymap = { u1 : 'here' }
        self.assertTrue(u1 in mymap)
        self.assertTrue(u1.uid() in mymap)
        self.assertEqual('here', mymap[u1.uid()])

    def testStringize (self):
        u1 = UniqueIdentifier('one')
        self.assertEqual('one', str(u1))

    def testRepr (self):
        u1 = UniqueIdentifier()
        rep = repr(u1)
        self.assertEqual('pyxb.utils.utility.UniqueIdentifier(%s)' % (repr(u1.uid()),), repr(u1))
        import pyxb.utils.utility
        u1b = eval(repr(u1))
        self.assertEqual(id(u1), id(u1b))

import os
import re
class TestGetMatchingFiles (unittest.TestCase):
    __WXS_re = re.compile('\.wxs$')
    __NoExt_re = re.compile('(^|\%s)[^\.]+$' % os.sep)
    __directory = None
    def setUp (self):
        self.__directory = tempfile.mkdtemp()
        #print 'setup %s' % (self.__directory,)

        dir_hierarchy = [ 'd1', 'd2', 'd3', 'd1/d11', 'd1/d12', 'd1/d12/d121' ]
        [ os.mkdir(os.path.join(self.__directory, _d)) for _d in dir_hierarchy ]
        files = [ 'd1/f1a.wxs', 'd1/f1b.wxs', 'd1/f1c',
                  'd2/f2a.wxs', 'd2/f2b',
                  'd3/f3a.wxs',
                  'd1/d11/f11a.wxs', 'd1/d11/f11b.wxs',
                  'd1/d12/f12a.wxs', 'd1/d12/f12b',
                  'd1/d12/d121/f121a.wxs' ]
        [ open(os.path.join(self.__directory, _f), 'w') for _f in files ]

        self.__haveSymlink = hasattr(os, 'symlink')
        if self.__haveSymlink:
            os.symlink(os.path.join(self.__directory, 'd2'), os.path.join(self.__directory, 'd1', 'd11', 'l2'))

    def tearDown (self):
        #print 'teardown %s' % (self.__directory,)
        for (root, dirs, files) in os.walk(self.__directory, False):
            [ os.unlink(os.path.join(root, _f)) for _f in files ]
            for d in dirs:
                dp = os.path.join(root, d)
                try:
                    os.rmdir(dp)
                except OSError:
                    os.unlink(dp)
        os.rmdir(self.__directory)

    def _formPath (self, *args):
        out_args = []
        for a in args:
            if '+' == a:
                out_args.append(a)
            else:
                out_args.append(os.path.join(self.__directory, a))
        return os.pathsep.join(out_args)

    def _stripPath (self, files):
        return [ _f[len(self.__directory)+1:].replace(os.sep, '/') for _f in files ]

    def testFormPath (self):
        # Make sure _formPath preserves trailing slashes
        saved_dir = self.__directory
        candidates = [ 'a', 'a/', 'a//', '+' ]
        try:
            self.__directory = os.path.join(os.sep, 'test')
            expanded = self._formPath(*candidates)
            expected = os.pathsep.join([os.path.join(self.__directory, 'a'),
                                        os.path.join(self.__directory, 'a', ''),
                                        os.path.join(self.__directory, 'a//'),
                                        '+'])
            self.assertEqual(expected, expanded)
            candidates.pop()
            expanded = self._formPath(*candidates)
            condensed = self._stripPath(expanded.split(os.pathsep))
            self.assertEqual(candidates, condensed)
        finally:
            self.__directory = saved_dir

    def testD1 (self):
        files = set(self._stripPath(GetMatchingFiles(self._formPath('d1'))))
        self.assertEqual(files, set(['d1/f1c', 'd1/f1b.wxs', 'd1/f1a.wxs']))

    def testPattern (self):
        files = set(self._stripPath(GetMatchingFiles(self._formPath('d1'), self.__WXS_re)))
        self.assertEqual(files, set(['d1/f1b.wxs', 'd1/f1a.wxs']))
        files = set(self._stripPath(GetMatchingFiles(self._formPath('d1'), self.__NoExt_re)))
        self.assertEqual(files, set(['d1/f1c']))
        files = set(self._stripPath(GetMatchingFiles(self._formPath('d1', 'd2'), self.__WXS_re)))
        self.assertEqual(files, set(['d1/f1a.wxs', 'd1/f1b.wxs', 'd2/f2a.wxs']))

    def testD1D2 (self):
        files = set(self._stripPath(GetMatchingFiles(self._formPath('d1', 'd2'))))
        self.assertEqual(files, set(['d1/f1a.wxs', 'd1/f1b.wxs', 'd1/f1c', 'd2/f2a.wxs', 'd2/f2b']))

    def testLink (self):
        if self.__haveSymlink:
            files = set(self._stripPath(GetMatchingFiles(self._formPath(os.path.join('d1', 'd11', 'l2')))))
            self.assertEqual(files, set(['d1/d11/l2/f2a.wxs', 'd1/d11/l2/f2b']))

    def testDefault (self):
        kw = { 'default_path' : self._formPath('d1'),
               'default_path_wildcard' : '+' }
        files = set(self._stripPath(GetMatchingFiles(self._formPath('+'), self.__WXS_re, **kw)))
        self.assertEqual(files, set(['d1/f1b.wxs', 'd1/f1a.wxs']))
        files = set(self._stripPath(GetMatchingFiles(self._formPath('d2', '+'), self.__WXS_re, **kw)))
        self.assertEqual(files, set(['d1/f1a.wxs', 'd1/f1b.wxs', 'd2/f2a.wxs']))
        files = set(self._stripPath(GetMatchingFiles(self._formPath('+', 'd2'), self.__WXS_re, **kw)))
        self.assertEqual(files, set(['d1/f1a.wxs', 'd1/f1b.wxs', 'd2/f2a.wxs']))

    def testPrefixPattern (self):
        kw = { 'prefix_pattern' : '&',
               'prefix_substituend' : self.__directory }
        # Note: &/d1 => /d1 so it does not count
        files = set(self._stripPath(GetMatchingFiles(os.pathsep.join([os.path.join('&', 'd1'), '&d2']), self.__NoExt_re, **kw)))
        self.assertEqual(files, set(['d2/f2b']))

    def testRecursive (self):
        files = set(self._stripPath(GetMatchingFiles(self._formPath('d1//'), self.__WXS_re)))
        self.assertEqual(files, set(['d1/f1a.wxs', 'd1/f1b.wxs', 'd1/d11/f11a.wxs', 'd1/d12/f12a.wxs', 'd1/d11/f11b.wxs', 'd1/d12/d121/f121a.wxs']))

    def testFiles (self):
        files = set(self._stripPath(GetMatchingFiles(self._formPath('d1', os.path.join('d2', 'f2b')), self.__NoExt_re)))
        self.assertEqual(files, set(['d1/f1c', 'd2/f2b']))

class TestIteratedCompareMixed (unittest.TestCase):

    def testBasicSameLen (self):
        lhs = (1, 2, 3)
        self.assertEqual(0, IteratedCompareMixed(lhs, (1, 2, 3)))
        self.assertEqual(-1, IteratedCompareMixed(lhs, (2, 2, 4)))
        self.assertEqual(1, IteratedCompareMixed(lhs, (0, 2, 2)))
        self.assertEqual(-1, IteratedCompareMixed(lhs, (1, 2, 4)))
        self.assertEqual(1, IteratedCompareMixed(lhs, (1, 2, 2)))

    def testMixedSameLen (self):
        lhs = (1, 'two', 3.2)
        self.assertEqual(0, IteratedCompareMixed(lhs, (1, 'two', 3.2)))
        self.assertEqual(-1, IteratedCompareMixed(lhs, (2, 'two', 3.2)))
        self.assertEqual(-1, IteratedCompareMixed(lhs, (1, 'txo', 3.2)))
        self.assertEqual(-1, IteratedCompareMixed(lhs, (1, 'two', 3.3)))
        self.assertEqual(1, IteratedCompareMixed(lhs, (0, 'two', 3.2)))
        self.assertEqual(1, IteratedCompareMixed(lhs, (1, 'three', 3.2)))
        self.assertEqual(1, IteratedCompareMixed(lhs, (1, 'two', 3.1)))
        rhs = lhs
        self.assertEqual(0, IteratedCompareMixed((1, 'two', 3.2), rhs))
        self.assertEqual(1, IteratedCompareMixed((2, 'two', 3.2), rhs))
        self.assertEqual(1, IteratedCompareMixed((1, 'txo', 3.2), rhs))
        self.assertEqual(1, IteratedCompareMixed((1, 'two', 3.3), rhs))
        self.assertEqual(-1, IteratedCompareMixed((0, 'two', 3.2), rhs))
        self.assertEqual(-1, IteratedCompareMixed((1, 'three', 3.2), rhs))
        self.assertEqual(-1, IteratedCompareMixed((1, 'two', 3.1), rhs))

    def testMixedNoneSameLen (self):
        lhs = (1, 'two', 3.2)
        self.assertEqual(0, IteratedCompareMixed(lhs, (1, 'two', 3.2)))
        self.assertEqual(1, IteratedCompareMixed(lhs, (None, 'two', 3.2)))
        self.assertEqual(1, IteratedCompareMixed(lhs, (1, None, 3.2)))
        self.assertEqual(1, IteratedCompareMixed(lhs, (1, 'two', None)))
        rhs = (1, 'two', 3.2)
        self.assertEqual(-1, IteratedCompareMixed((None, 'two', 3.2), rhs))
        self.assertEqual(-1, IteratedCompareMixed((1, None, 3.2), rhs))
        self.assertEqual(-1, IteratedCompareMixed((1, 'two', None), rhs))

    def testDifferentLen (self):
        self.assertEqual(-1, IteratedCompareMixed((1, 2, 3), (1, 2, 3, -1)))
        self.assertEqual(1, IteratedCompareMixed((1, 2, 3, -1), (1, 2, 3)))

if '__main__' == __name__:
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.binding.datatypes as xs
import xml.dom

import os.path
xsd='''<?xml version="1.0" encoding="UTF-8"?>

<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
  <xs:simpleType name="stype">
     <xs:restriction base="xs:string"/>
  </xs:simpleType>
  <xs:element name="selt" type="stype"/>
  <xs:complexType name="wrapper">
    <xs:sequence>
      <xs:element name="first" minOccurs="0"/>
      <xs:element name="second" minOccurs="0"/>
      <xs:any namespace="##any" minOccurs="0" maxOccurs="3" processContents="lax"/>
    </xs:sequence>
    <xs:attribute name="myattr" use="optional" type="xs:boolean"/>
    <xs:anyAttribute namespace="##any" processContents="lax"/>
  </xs:complexType>
  <xs:element name="wrapper" type="wrapper"/>
</xs:schema>
'''

#open('schema.xsd', 'w').write(xsd)
code = pyxb.binding.generate.GeneratePython(schema_text=xsd)
#open('code.py', 'w').write(code)

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

class TestTrac_0137 (unittest.TestCase):
    def setUp (self):
        # Hide the warning about failure to convert DOM node {}third
        # to a binding
        self.__basis_log = logging.getLogger('pyxb.binding.basis')
        self.__basis_loglevel = self.__basis_log.level
        self.__basis_log.setLevel(logging.ERROR)

    def tearDown (self):
        pyxb.RequireValidWhenParsing(True)
        self.__basis_log.level = self.__basis_loglevel

    def validate (self):
        xmls = '<wrapper><first/><second/><third><selt>text</selt></third></wrapper>'
        doc = pyxb.utils.domutils.StringToDOM(xmls)
        instance = wrapper.createFromDOM(doc.documentElement)
        self.assertEqual(1, len(instance.wildcardElements()))
        third = instance.wildcardElements()[0]
        self.assertTrue(isinstance(third, xml.dom.Node))
        self.assertEqual(xml.dom.Node.ELEMENT_NODE, third.nodeType)
        self.assertEqual('third', third.localName)

    def testWithValidation (self):
        pyxb.RequireValidWhenParsing(True)
        self.validate()

    def testWithoutValidation (self):
        pyxb.RequireValidWhenParsing(False)
        self.validate()

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import unittest
import pyxb.binding.datatypes as xsd

class Test_double (unittest.TestCase):
    def testRange (self):
        # Not going to do anything for this
        pass

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import xml.dom.minidom
import pyxb.utils.domutils

import unittest

class TestTrac0156a (unittest.TestCase):
    def testLocateUnique (self):
        dom = xml.dom.minidom.parseString('<xs:simpleType xmlns:xs="http://www.w3.org/2001/XMLSchema"><xs:list>x</xs:list></xs:simpleType>')
        x = pyxb.utils.domutils.LocateUniqueChild (dom.firstChild, 'list')
        dom = xml.dom.minidom.parseString('<xs:simpleType xmlns:xs="http://www.w3.org/2001/XMLSchema"><xs:list>x</xs:list><xs:list>x</xs:list></xs:simpleType>')
        self.assertRaises(pyxb.SchemaValidationError, pyxb.utils.domutils.LocateUniqueChild, dom.firstChild, 'list')
        dom = xml.dom.minidom.parseString('<xs:simpleType xmlns:xs="http://www.w3.org/2001/XMLSchema"><xs:lit>x</xs:lit></xs:simpleType>')
        self.assertRaises(pyxb.SchemaValidationError, pyxb.utils.domutils.LocateUniqueChild, dom.firstChild, 'list', absent_ok=False)

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.utils.domutils
from pyxb.utils import six
from xml.dom import Node

import os.path
xst = six.u('''<?xml version="1.0" encoding="UTF-8"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
 <xs:complexType name="XsdWithHyphens">
    <xs:sequence>
        <xs:element name="username" type="xs:string"/>
    </xs:sequence>
  </xs:complexType>
  <xs:element name="xsd-with-hyphens" type="XsdWithHyphens"/>
</xs:schema>
''')

code = pyxb.binding.generate.GeneratePython(schema_text=xst)
#print code

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

class TestTrac0203 (unittest.TestCase):
    def testBasic (self):
        unbound = XsdWithHyphens('name')
        if sys.version_info[:2] < (2, 7):
            self.assertRaises(pyxb.UnboundElementError, unbound.toxml, 'utf-8', root_only=True)
        else:
            with self.assertRaises(pyxb.UnboundElementError) as cm:
                unbound.toxml('utf-8', root_only=True)
            e = cm.exception
            self.assertEqual(e.instance, unbound)
            self.assertEqual('Instance of type XsdWithHyphens has no bound element for start tag', str(e))

    def testOverride (self):
        unbound = XsdWithHyphens('name')
        xmlt = six.u('<root><username>name</username></root>')
        xmld = xmlt.encode('utf-8')
        self.assertEqual(xmld, unbound.toxml('utf-8', root_only=True, element_name='root'))

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb
import pyxb.binding.generate
import pyxb.utils.domutils
from pyxb.utils import six
from xml.dom import Node

import pyxb.binding.basis
import os.path
schema_path = os.path.abspath(os.path.join(os.path.dirname(__file__), '../schemas/test-collision.xsd'))
code = pyxb.binding.generate.GeneratePython(schema_location=schema_path)
#open('code.py', 'w').write(code)

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.utils import domutils

import unittest

class TestCollision (unittest.TestCase):

    def testBasic (self):
        instance = color(color_.red, color_=color_.green)
        xmlt = six.u('<color color="green"><color>red</color></color>')
        xmld = xmlt.encode('utf-8')
        self.assertEqual(instance.toxml("utf-8", root_only=True), xmld)
        instance.color = color_.blue
        xmlt = six.u('<color color="green"><color>blue</color></color>')
        xmld = xmlt.encode('utf-8')
        self.assertEqual(instance.toxml("utf-8", root_only=True), xmld)
        instance.color_ = color_.red
        xmlt = six.u('<color color="red"><color>blue</color></color>')
        xmld = xmlt.encode('utf-8')
        self.assertEqual(instance.toxml("utf-8", root_only=True), xmld)

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import unittest
import pyxb.binding.datatypes as xsd

class Test_gMonthDay (unittest.TestCase):
    def testBasic (self):
        v = xsd.gMonthDay('--10-27')
        #self.assertEqual(v.year, 2002)
        self.assertEqual(v.month, 10)
        self.assertEqual(v.day, 27)
        v = xsd.gMonthDay(10, 27)
        #self.assertEqual(v.year, 2002)
        self.assertEqual(v.month, 10)
        self.assertEqual(v.day, 27)
        self.assertRaises(TypeError, xsd.gMonthDay, 2002)

    def testXSDLiteral (self):
        v = xsd.gMonthDay(10, 27)
        self.assertEqual('--10-27', v.xsdLiteral())


    def testTimezoned (self):
        dt = xsd.gMonthDay('--10-10Z')
        self.assertEqual('1900-10-10T00:00:00+00:00', dt.isoformat())
        self.assertEqual('--10-10Z', dt.xsdLiteral())

        dt = xsd.gMonthDay('--10-10+13:00')
        self.assertEqual('1900-10-10T00:00:00+13:00', dt.isoformat())
        self.assertEqual('--10-10+13:00', dt.xsdLiteral())
        dt = xsd.gMonthDay('--10-09-11:00')
        self.assertEqual('1900-10-09T00:00:00-11:00', dt.isoformat())
        self.assertEqual('--10-09-11:00', dt.xsdLiteral())

        dt = xsd.gMonthDay('--10-10+14:00')
        self.assertEqual('1900-10-10T00:00:00+14:00', dt.isoformat())
        self.assertEqual('--10-10+14:00', dt.xsdLiteral())
        dt = xsd.gMonthDay('--10-09-10:00')
        self.assertEqual('1900-10-09T00:00:00-10:00', dt.isoformat())
        self.assertEqual('--10-09-10:00', dt.xsdLiteral())

        dt = xsd.gMonthDay('--10-10-14:00')
        self.assertEqual('1900-10-10T00:00:00-14:00', dt.isoformat())
        self.assertEqual('--10-10-14:00', dt.xsdLiteral())
        dt = xsd.gMonthDay('--10-11+10:00')
        self.assertEqual('1900-10-11T00:00:00+10:00', dt.isoformat())
        self.assertEqual('--10-11+10:00', dt.xsdLiteral())

    def XtestAccessor (self):
        v = xsd.gMonthDay(10, 27)
        self.assertRaises((AttributeError, TypeError), getattr, v, 'year')
        #self.assertRaises((AttributeError, TypeError), getattr, v, 'month')
        #self.assertRaises((AttributeError, TypeError), getattr, v, 'day')
        self.assertRaises((AttributeError, TypeError), setattr, v, 'year', 5)
        self.assertRaises((AttributeError, TypeError), setattr, v, 'month', 5)
        self.assertRaises((AttributeError, TypeError), setattr, v, 'day', 5)

if __name__ == '__main__':
    unittest.main()



from __future__ import print_function
from pyxb import BIND
import content

v = content.numbers(1, BIND(2), attribute=3)
v.complex.style = "decimal"
print(v.toxml("utf-8").decode('utf-8'))
print(3 * v.simple)
print(4 * v.complex.value())
print(5 * v.attribute)



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
from pyxb.utils.domutils import BindingDOMSupport
from pyxb.utils import six
import unittest

import bindings._A as A

class Test (unittest.TestCase):
    def setUp (self):
        BindingDOMSupport.DeclareNamespace(A.Namespace, 'a')

    def tearDown (self):
        BindingDOMSupport.Reset()

    def tests (self):
        x = A.A_c_e1("A_b_e1", "e1")
        xmlt = six.u('<a:A_c_e1 xmlns:a="URN:nsdep:A"><a:A_b_e1>A_b_e1</a:A_b_e1><a:e1>e1</a:e1></a:A_c_e1>')
        xmld = xmlt.encode('utf-8')
        self.assertEqual(x.toxml("utf-8", root_only=True), xmld)

if '__main__' == __name__:
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
from pyxb.exceptions_ import *
import unittest
import pyxb.binding.datatypes as xsd

class Test_normalizedString (unittest.TestCase):
    Cases = [ ("with\nnewline", 'with newline'),
              ("with\rreturn", 'with return'),
              ("with\ttab", 'with tab'),
              ("\n\nleading newline", '  leading newline'),
              ("trailing newline\n\n", 'trailing newline  '),
              ]

    def testValid (self):
        for (lexical, value) in self.Cases:
            self.assertEqual(value, xsd.normalizedString(value))
            self.assertEqual(value, xsd.normalizedString(lexical, _from_xml=True))

    def testInvalid (self):
        for (lexical, value) in self.Cases:
            self.assertRaises(SimpleTypeValueError, xsd.normalizedString, lexical)

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import sys
import unittest

class TestTrac0099a (unittest.TestCase):
    def setUp (self):
        # Hide the error about failure to check value constraint
        self.__structures_log = logging.getLogger('pyxb.xmlschema.structures')
        self.__structures_loglevel = self.__structures_log.level
        self.__structures_log.setLevel(logging.CRITICAL)

    def tearDown (self):
        pyxb.RequireValidWhenParsing(True)
        self.__structures_log.level = self.__structures_loglevel

    def testRefWithAttrs (self):
        xsd = '''<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
  <xs:element name="uncs" type="xs:string"/>
  <xs:element name="complex">
    <xs:complexType>
      <xs:sequence>
        <xs:element ref="uncs" default="value"/>
      </xs:sequence>
    </xs:complexType>
  </xs:element>
</xs:schema>'''
        self.assertRaises(pyxb.SchemaValidationError, pyxb.binding.generate.GeneratePython, schema_text=xsd)

    def testConstraintOnComplex (self):
        xsd = '''<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
  <xs:element name="complex" default="foo">
    <xs:complexType>
      <xs:sequence>
        <xs:element name="elt" type="xs:string"/>
      </xs:sequence>
    </xs:complexType>
  </xs:element>
</xs:schema>'''
        if sys.version_info[:2] < (2, 7):
            self.assertRaises(pyxb.SchemaValidationError, pyxb.binding.generate.GeneratePython, schema_text=xsd)
            return
        with self.assertRaises(pyxb.SchemaValidationError) as cm:
            code = pyxb.binding.generate.GeneratePython(schema_text=xsd)
        self.assertEqual('Value constraint on element complex with non-simple content', str(cm.exception))

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import unittest

class TestTrac0161 (unittest.TestCase):

    def testBasicGenerator (self):
        g = pyxb.binding.generate.Generator()
        args = g.getCommandLineArgs()

    def testPreloadPath (self):
        g = pyxb.binding.generate.Generator(pre_load_archives=['something.wxs'])
        args = g.getCommandLineArgs()

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.binding.datatypes as xs
import pyxb.binding.basis
import pyxb.utils.domutils

import os.path
xsd='''
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
  <xs:complexType name="BaseT" abstract="true"/>
  <xs:complexType name="ChildT">
     <xs:complexContent>
        <xs:extension base="BaseT">
           <xs:sequence>
             <xs:element name="kid" type="xs:string"/>
           </xs:sequence>
        </xs:extension>
     </xs:complexContent>
  </xs:complexType>

  <xs:element name="NotAType" type="xs:string"/>
  <xs:element name="Child">
     <xs:complexContent>
        <xs:extension base="BaseT">
           <xs:sequence minOccurs="0">
             <xs:element name="kid2" type="xs:string"/>
           </xs:sequence>
        </xs:extension>
     </xs:complexContent>
  </xs:element>

</xs:schema>
'''

#open('schema.xsd', 'w').write(xsd)
code = pyxb.binding.generate.GeneratePython(schema_text=xsd)
#open('code.py', 'w').write(code)
#print code

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

class TestTrac_0056 (unittest.TestCase):
    def testNonType (self):
        xmls = '<Child xsi:type="NotAType" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"/>'
        self.assertRaises(pyxb.BadDocumentError, CreateFromDocument, xmls)
        doc = pyxb.utils.domutils.StringToDOM(xmls)
        self.assertRaises(pyxb.BadDocumentError, CreateFromDOM, doc)

    def testAnonymousBase (self):
        xmls = '<Child xsi:type="ChildT" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"/>'
        #self.assertRaises(pyxb.BadDocumentError, CreateFromDocument, xmls)
        doc = pyxb.utils.domutils.StringToDOM(xmls)
        self.assertRaises(pyxb.ValidationError, CreateFromDOM, doc)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import sys
import pyxb.binding.generate
import pyxb.utils.domutils


xmls = '''<?xml version="1.0"?>
<Signature xmlns="http://www.w3.org/2000/09/xmldsig#">
  <SignedInfo>
    <CanonicalizationMethod Algorithm="http://www.w3.org/2001/10/xml-exc-c14n#"/>
    <SignatureMethod Algorithm="http://www.w3.org/2000/09/xmldsig#rsa-sha1"/>
    <Reference URI="#_53cdd0e9-bd51-4a49-8519-f711860c5499">
      <Transforms>
        <Transform Algorithm="http://www.w3.org/2000/09/xmldsig#enveloped-signature"/>
        <Transform Algorithm="http://www.w3.org/2001/10/xml-exc-c14n#">
          <InclusiveNamespaces xmlns="http://www.w3.org/2001/10/xml-exc-c14n#" PrefixList="#default samlp saml ds xs xsi"/>
        </Transform>
      </Transforms>
      <DigestMethod Algorithm="http://www.w3.org/2000/09/xmldsig#sha1"/>
      <DigestValue>4OXjC8LT68JYyCVbtKJims+qe4U=</DigestValue>
    </Reference>
  </SignedInfo>
  <SignatureValue>vG+UCB9C3T8k91yDP2zCvKNHJbp+P1aik9xckByr43y5V5BZktzOF9h4FC8C3mYklg/jTtzWiN4TvAd+wrznpOYDBv3/AseHM+LWG2Q/0t1o8owiliR1z8BqydQUXPuwTSTrPAzEmYFKnq+OQvtq8GiebfYHD2nTKc4M0B8//TuQ295WLwX06RXiuNxGN9C1YvMOL/hHKybPeiVbT7I0wnUMDjf4H/K+4hnCZY2wT+nOBf0fRVkFud/0/lIxCu8T3SMQeSzMRXcdK0FiElFNzh24DJerVwIIZTFXl5Qg42S2Is6kFs1KR0CYDXR/ZVNF5CdVY/xe62t8GUYbTHAoBw==</SignatureValue>
</Signature>'''

import unittest

class TestTrac0123 (unittest.TestCase):
    def testEmpty (self):
        try:
            import pyxb.bundles.wssplat.ds as dsig
        except ImportError as e:
            _log.warning('%s: skipping test, error importing dsig', __file__)
            return
        # The warning that InclusiveNamespaces could not be converted
        # is correct.  We don't have bindings for
        # http://www.w3.org/2001/10/xml-exc-c14n#.  We're going to
        # leave it that way for now, as evidence the warning is still
        # present.
        instance = dsig.CreateFromDocument(xmls)
        # Problem was the PrefixList attribute in InclusiveNamespaces,
        # which has no namespace; deepClone couldn't handle it.  Show
        # that it can.
        dom = instance.toDOM()

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
# Copyright 2009-2013, Peter A. Bigot
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain a
# copy of the License at:
#
#            http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""Classes and global objects related to resolving U{XML
Namespaces<http://www.w3.org/TR/2006/REC-xml-names-20060816/index.html>}."""

import logging
import pyxb
import pyxb.utils.utility
from pyxb.namespace import archive, utility
from pyxb.utils import six

_log = logging.getLogger(__name__)

class _Resolvable_mixin (pyxb.cscRoot):
    """Mix-in indicating that this object may have references to unseen named components.

    This class is mixed-in to those XMLSchema components that have a reference
    to another component that is identified by a QName.  Resolution of that
    component may need to be delayed if the definition of the component has
    not yet been read.
    """

    #_TraceResolution = True
    _TraceResolution = False

    def isResolved (self):
        """Determine whether this named component is resolved.

        Override this in the child class."""
        raise NotImplementedError("_Resolvable_mixin.isResolved in %s"% (type(self).__name__,))

    def _resolve (self):
        """Perform whatever steps are required to resolve this component.

        Resolution is performed in the context of the namespace to which the
        component belongs.  Invoking this method may fail to complete the
        resolution process if the component itself depends on unresolved
        components.  The sole caller of this should be
        L{_NamespaceResolution_mixin.resolveDefinitions}.

        This method is permitted (nay, encouraged) to raise an exception if
        resolution requires interpreting a QName and the named component
        cannot be found.

        Override this in the child class.  In the prefix, if L{isResolved} is
        true, return right away.  If something prevents you from completing
        resolution, invoke L{self._queueForResolution()} (so it is retried
        later) and immediately return self.  Prior to leaving after successful
        resolution discard any cached dom node by setting C{self.__domNode=None}.

        @return: C{self}, whether or not resolution succeeds.
        @raise pyxb.SchemaValidationError: if resolution requlres a reference to an unknown component
        """
        raise NotImplementedError("_Resolvable_mixin._resolve in %s"% (type(self).__name__,))

    def _queueForResolution (self, why=None, depends_on=None):
        """Short-hand to requeue an object if the class implements _namespaceContext().
        """
        if (why is not None) and self._TraceResolution:
            _log.info('Resolution delayed for %s: %s\n\tDepends on: %s', self, why, depends_on)
        self._namespaceContext().queueForResolution(self, depends_on)

class _NamespaceResolution_mixin (pyxb.cscRoot):
    """Mix-in that aggregates those aspects of XMLNamespaces relevant to
    resolving component references.
    """

    # A set of namespaces which some schema imported while processing with
    # this namespace as target.
    __importedNamespaces = None

    # A set of namespaces which appear in namespace declarations of schema
    # with this namespace as target.
    __referencedNamespaces = None

    # A list of Namespace._Resolvable_mixin instances that have yet to be
    # resolved.
    __unresolvedComponents = None

    # A map from Namespace._Resolvable_mixin instances in
    # __unresolvedComponents to sets of other unresolved objects on which they
    # depend.
    __unresolvedDependents = None

    def _reset (self):
        """CSC extension to reset fields of a Namespace.

        This one handles component-resolution--related data."""
        getattr(super(_NamespaceResolution_mixin, self), '_reset', lambda *args, **kw: None)()
        self.__unresolvedComponents = []
        self.__unresolvedDependents = {}
        self.__importedNamespaces = set()
        self.__referencedNamespaces = set()

    def _getState_csc (self, kw):
        kw.update({
                'importedNamespaces': self.__importedNamespaces,
                'referencedNamespaces': self.__referencedNamespaces,
                })
        return getattr(super(_NamespaceResolution_mixin, self), '_getState_csc', lambda _kw: _kw)(kw)

    def _setState_csc (self, kw):
        self.__importedNamespaces = kw['importedNamespaces']
        self.__referencedNamespaces = kw['referencedNamespaces']
        return getattr(super(_NamespaceResolution_mixin, self), '_setState_csc', lambda _kw: self)(kw)

    def importNamespace (self, namespace):
        self.__importedNamespaces.add(namespace)
        return self

    def _referenceNamespace (self, namespace):
        self._activate()
        self.__referencedNamespaces.add(namespace)
        return self

    def importedNamespaces (self):
        """Return the set of namespaces which some schema imported while
        processing with this namespace as target."""
        return frozenset(self.__importedNamespaces)

    def _transferReferencedNamespaces (self, module_record):
        assert isinstance(module_record, archive.ModuleRecord)
        module_record._setReferencedNamespaces(self.__referencedNamespaces)
        self.__referencedNamespaces.clear()

    def referencedNamespaces (self):
        """Return the set of namespaces which appear in namespace declarations
        of schema with this namespace as target."""
        return frozenset(self.__referencedNamespaces)

    def queueForResolution (self, resolvable, depends_on=None):
        """Invoked to note that a component may have references that will need
        to be resolved.

        Newly created named components are often unresolved, as are components
        which, in the course of resolution, are found to depend on another
        unresolved component.

        @param resolvable: An instance of L{_Resolvable_mixin} that is later to
        be resolved.

        @keyword depends_on: C{None}, or an instance of L{_Resolvable_mixin}
        which C{resolvable} requires to be resolved in order to resolve
        itself.

        @return: C{resolvable}
        """
        assert isinstance(resolvable, _Resolvable_mixin)
        if not resolvable.isResolved():
            assert depends_on is None or isinstance(depends_on, _Resolvable_mixin)
            self.__unresolvedComponents.append(resolvable)
            if depends_on is not None and not depends_on.isResolved():
                from pyxb.xmlschema import structures
                assert isinstance(depends_on, _Resolvable_mixin)
                assert isinstance(depends_on, structures._NamedComponent_mixin)
                self.__unresolvedDependents.setdefault(resolvable, set()).add(depends_on)
        return resolvable

    def needsResolution (self):
        """Return C{True} iff this namespace has not been resolved."""
        return self.__unresolvedComponents is not None

    def _replaceComponent_csc (self, existing_def, replacement_def):
        """Replace a component definition if present in the list of unresolved components.
        """
        try:
            index = self.__unresolvedComponents.index(existing_def)
            if (replacement_def is None) or (replacement_def in self.__unresolvedComponents):
                del self.__unresolvedComponents[index]
            else:
                assert isinstance(replacement_def, _Resolvable_mixin)
                self.__unresolvedComponents[index] = replacement_def
            # Rather than assume the replacement depends on the same
            # resolvables as the original, just wipe the dependency record:
            # it'll get recomputed later if it's still important.
            if existing_def in self.__unresolvedDependents:
                del self.__unresolvedDependents[existing_def]
        except ValueError:
            pass
        return getattr(super(_NamespaceResolution_mixin, self), '_replaceComponent_csc', lambda *args, **kw: replacement_def)(existing_def, replacement_def)

    def resolveDefinitions (self, allow_unresolved=False):
        """Loop until all references within the associated resolvable objects
        have been resolved.

        This method iterates through all components on the unresolved list,
        invoking the _resolve method of each.  If the component could not be
        resolved in this pass, it iis placed back on the list for the next
        iteration.  If an iteration completes without resolving any of the
        unresolved components, a pyxb.NotInNamespaceError exception is raised.

        @note: Do not invoke this until all top-level definitions for the
        namespace have been provided.  The resolution routines are entitled to
        raise a validation exception if a reference to an unrecognized
        component is encountered.
        """
        if not self.needsResolution():
            return True

        while 0 < len(self.__unresolvedComponents):
            # Save the list of unresolved objects, reset the list to capture
            # any new objects defined during resolution, and attempt the
            # resolution for everything that isn't resolved.
            unresolved = self.__unresolvedComponents

            self.__unresolvedComponents = []
            self.__unresolvedDependents = {}
            for resolvable in unresolved:
                # Attempt the resolution.
                resolvable._resolve()

                # Either we resolved it, or we queued it to try again later
                assert resolvable.isResolved() or (resolvable in self.__unresolvedComponents), 'Lost resolvable %s' % (resolvable,)

                # We only clone things that have scope None.  We never
                # resolve things that have scope None.  Therefore, we
                # should never have resolved something that has
                # clones.
                if (resolvable.isResolved() and (resolvable._clones() is not None)):
                    assert False
            if self.__unresolvedComponents == unresolved:
                if allow_unresolved:
                    return False
                # This only happens if we didn't code things right, or the
                # there is a circular dependency in some named component
                # (i.e., the schema designer didn't do things right).
                failed_components = []
                from pyxb.xmlschema import structures
                for d in self.__unresolvedComponents:
                    if isinstance(d, structures._NamedComponent_mixin):
                        failed_components.append('%s named %s' % (d.__class__.__name__, d.name()))
                    else:
                        failed_components.append('Anonymous %s' % (d.__class__.__name__,))
                raise pyxb.NotInNamespaceError('Infinite loop in resolution:\n  %s' % ("\n  ".join(failed_components),))

        # Replace the list of unresolved components with None, so that
        # attempts to subsequently add another component fail.
        self.__unresolvedComponents = None
        self.__unresolvedDependents = None

        # NOTE: Dependencies may require that we keep these around for a while
        # longer.
        #
        # Remove the namespace context from everything, since we won't be
        # resolving anything else.
        self._releaseNamespaceContexts()

        return True

    def _unresolvedComponents (self):
        """Returns a reference to the list of unresolved components."""
        return self.__unresolvedComponents

    def _unresolvedDependents (self):
        """Returns a map from unresolved components to sets of components that
        must be resolved first."""
        return self.__unresolvedDependents

def ResolveSiblingNamespaces (sibling_namespaces):
    """Resolve all components in the sibling_namespaces.

    @param sibling_namespaces : A set of namespaces expected to be closed
    under dependency."""

    for ns in sibling_namespaces:
        ns.configureCategories([archive.NamespaceArchive._AnonymousCategory()])
        ns.validateComponentModel()

    def __keyForCompare (dependency_map):
        """Sort namespaces so dependencies get resolved first.

        Uses the trick underlying functools.cmp_to_key(), but optimized for
        this special case.  The dependency map is incorporated into the class
        definition by scope.
        """
        class K (object):
            def __init__ (self, ns, *args):
                self.__ns = ns

            # self compares less than other if self.ns is in the dependency set
            # of other.ns but not vice-versa.
            def __lt__ (self, other):
                return ((self.__ns in dependency_map.get(other.__ns, set())) \
                            and not (other.__ns in dependency_map.get(self.__ns, set())))

            # self compares equal to other if their namespaces are either
            # mutually dependent or independent.
            def __eq__ (self, other):
                return (self.__ns in dependency_map.get(other.__ns, set())) == (other.__ns in dependency_map.get(self.__ns, set()))

            # All other order metrics are derived.
            def __ne__ (self, other):
                return not self.__eq__(other)
            def __le__ (self, other):
                return self.__lt__(other) or self.__eq__(other)
            def __gt__ (self, other):
                return other.__lt__(self.__ns)
            def __ge__ (self, other):
                return other.__lt__(self.__ns) or self.__eq__(other)
        return K

    need_resolved_set = set(sibling_namespaces)
    dependency_map = {}
    last_state = None
    while need_resolved_set:
        need_resolved_list = list(need_resolved_set)
        if dependency_map:
            need_resolved_list.sort(key=__keyForCompare(dependency_map))
        need_resolved_set = set()
        dependency_map = {}
        for ns in need_resolved_list:
            if not ns.needsResolution():
                continue
            if not ns.resolveDefinitions(allow_unresolved=True):
                deps = dependency_map.setdefault(ns, set())
                for (c, dcs) in six.iteritems(ns._unresolvedDependents()):
                    for dc in dcs:
                        dns = dc.expandedName().namespace()
                        if dns != ns:
                            deps.add(dns)
                _log.info('Holding incomplete resolution %s depending on: ', ns.uri(), six.u(' ; ').join([ six.text_type(_dns) for _dns in deps ]))
                need_resolved_set.add(ns)
        # Exception termination check: if we have the same set of incompletely
        # resolved namespaces, and each has the same number of unresolved
        # components, assume there's an truly unresolvable dependency: either
        # due to circularity, or because there was an external namespace that
        # was missed from the sibling list.
        state = []
        for ns in need_resolved_set:
            state.append( (ns, len(ns._unresolvedComponents())) )
        state = tuple(state)
        if last_state == state:
            raise pyxb.LogicError('Unexpected external dependency in sibling namespaces: %s' % (six.u('\n  ').join( [six.text_type(_ns) for _ns in need_resolved_set ]),))
        last_state = state

@six.python_2_unicode_compatible
class NamespaceContext (object):
    """Records information associated with namespaces at a DOM node.
    """

    def __str__ (self):
        rv = [ six.u('NamespaceContext ') ]
        if self.defaultNamespace() is not None:
            rv.extend([ '(defaultNamespace=', six.text_type(self.defaultNamespace()), ') '])
        if self.targetNamespace() is not None:
            rv.extend([ '(targetNamespace=', six.text_type(self.targetNamespace()), ') '])
        rv.append("\n")
        for (pfx, ns) in six.iteritems(self.inScopeNamespaces()):
            if pfx is not None:
                rv.append('  xmlns:%s=%s' % (pfx, six.text_type(ns)))
        return six.u('').join(rv)

    __ContextStack = []
    @classmethod
    def PushContext (cls, ctx):
        """Make C{ctx} the currently active namespace context.

        Prior contexts are retained on a LIFO stack."""
        assert isinstance(ctx, cls)
        cls.__ContextStack.append(ctx)
        return ctx

    @classmethod
    def Current (cls):
        """Access the currently active namespace context.

        If no context is active, C{None} is returned.  This probably
        represents mis-use of the infrastructure (viz., failure to record the
        context within which a QName must be resolved)."""
        if cls.__ContextStack:
            return cls.__ContextStack[-1]
        return None

    @classmethod
    def PopContext (cls):
        """Discard the currently active namespace context, restoring its
        predecessor.

        The discarded context is returned."""
        return cls.__ContextStack.pop()

    __TargetNamespaceAttributes = { }
    @classmethod
    def _AddTargetNamespaceAttribute (cls, expanded_name, attribute_name):
        assert expanded_name is not None
        cls.__TargetNamespaceAttributes[expanded_name] = attribute_name
    @classmethod
    def _TargetNamespaceAttribute (cls, expanded_name):
        return cls.__TargetNamespaceAttributes.get(expanded_name)

    # Support for holding onto referenced namespaces until we have a target
    # namespace to give them to.
    __pendingReferencedNamespaces = None

    def defaultNamespace (self):
        """The default namespace in effect at this node.  E.g., C{xmlns="URN:default"}."""
        return self.__defaultNamespace
    __defaultNamespace = None

    def setDefaultNamespace (self, default_namespace):
        """Set the default namespace for the generated document.

        Even if invoked post construction, the default namespace will affect
        the entire document, as all namespace declarations are placed in the
        document root.

        @param default_namespace: The namespace to be defined as the default
        namespace in the top-level element of the document.  May be provided
        as a real namespace, or just its URI.
        @type default_namespace: L{pyxb.namespace.Namespace} or C{str} or
        C{unicode}.
        """

        if isinstance(default_namespace, six.string_types):
            default_namespace = utility.NamespaceForURI(default_namespace, create_if_missing=True)
        if (default_namespace is not None) and default_namespace.isAbsentNamespace():
            raise pyxb.UsageError('Default namespace must not be an absent namespace')
        self.__defaultNamespace = default_namespace

    # If C{True}, this context is within a schema that has no target
    # namespace, and we should use the target namespace as a fallback if no
    # default namespace is available and no namespace prefix appears on a
    # QName.  This situation arises when a top-level schema has an absent
    # target namespace, or when a schema with an absent target namespace is
    # being included into a schema with a non-absent target namespace.
    __fallbackToTargetNamespace = False

    def targetNamespace (self):
        """The target namespace in effect at this node.  Usually from the
        C{targetNamespace} attribute.  If no namespace is specified for the
        schema, an absent namespace was assigned upon creation and will be
        returned."""
        return self.__targetNamespace
    __targetNamespace = None

    def inScopeNamespaces (self):
        """Map from prefix strings to L{Namespace} instances associated with those
        prefixes.  The prefix C{None} identifies the default namespace."""
        return self.__inScopeNamespaces
    __inScopeNamespaces = None

    """Map from L{Namespace} instances to sets of prefix strings associated
    with the namespace.  The default namespace is not represented."""
    __inScopePrefixes = None

    def __removePrefixMap (self, pfx):
        ns = self.__inScopeNamespaces.pop(pfx, None)
        if ns is not None:
            pfxs = self.__inScopePrefixes.get(ns)
            if pfxs is not None:
                pfxs.discard(pfx)

    def __addPrefixMap (self, pfx, ns):
        # Any previous assignment must have already been removed
        self.__inScopeNamespaces[pfx] = ns
        self.__inScopePrefixes.setdefault(ns, set()).add(pfx)

    def __clonePrefixMap (self):
        self.__inScopeNamespaces = self.__inScopeNamespaces.copy()
        isp = {}
        for (ns, pfxs) in six.iteritems(self.__inScopePrefixes):
            isp[ns] = pfxs.copy()
        self.__inScopePrefixes = isp

    # Class-scope initial map from prefix to namespace
    __InitialScopeNamespaces = None
    # Class-scope initial map from namespace to prefix(es)
    __InitialScopePrefixes = None
    # Instance-specific initial map from prefix to namespace
    __initialScopeNamespaces = None
    # Instance-specific initial map from namespace to prefix(es)
    __initialScopePrefixes = None


    @classmethod
    def __BuildInitialPrefixMap (cls):
        if cls.__InitialScopeNamespaces is not None:
            return
        from pyxb.namespace import builtin
        cls.__InitialScopeNamespaces = builtin._UndeclaredNamespaceMap
        cls.__InitialScopePrefixes = {}
        for (pfx, ns) in six.iteritems(cls.__InitialScopeNamespaces):
            cls.__InitialScopePrefixes.setdefault(ns, set()).add(pfx)

    def prefixForNamespace (self, namespace):
        """Return a prefix associated with the given namespace in this
        context, or None if the namespace is the default or is not in
        scope."""
        pfxs = self.__inScopePrefixes.get(namespace)
        if pfxs:
            return next(iter(pfxs))
        return None

    @classmethod
    def GetNodeContext (cls, node, **kw):
        """Get the L{NamespaceContext} instance that was assigned to the node.

        If none has been assigned and keyword parameters are present, create
        one treating this as the root node and the keyword parameters as
        configuration information (e.g., default_namespace).

        @raise pyxb.LogicError: no context is available and the keywords
        required to create one were not provided
        """
        try:
            return node.__namespaceContext
        except AttributeError:
            return NamespaceContext(node, **kw)

    def setNodeContext (self, node):
        node.__namespaceContext = self

    # Integer counter to help generate unique namespace prefixes
    __namespacePrefixCounter = None

    def declareNamespace (self, namespace, prefix=None, add_to_map=False):
        """Record the given namespace as one to be used in this document.

        @param namespace: The namespace to be associated with the document.
        @type namespace: L{pyxb.namespace.Namespace}

        @keyword prefix: Optional prefix to be used with this namespace.  If
        not provided, a unique prefix is generated or a standard prefix is
        used, depending on the namespace.

        @return: a prefix that may be used with the namespace.  If C{prefix}
        was C{None} the return value may be a previously-assigned prefix.

        @todo: ensure multiple namespaces do not share the same prefix
        @todo: provide default prefix in L{pyxb.namespace.Namespace}
        """
        if not isinstance(namespace, pyxb.namespace.Namespace):
            raise pyxb.UsageError('declareNamespace: must be given a namespace instance')
        if namespace.isAbsentNamespace():
            raise pyxb.UsageError('declareNamespace: namespace must not be an absent namespace')
        if prefix is None:
            prefix = namespace.prefix()
        if prefix is None:
            pfxs = self.__inScopePrefixes.get(namespace)
            if pfxs:
                prefix = next(iter(pfxs))
        while prefix is None:
            self.__namespacePrefixCounter += 1
            candidate_prefix = 'ns%d' % (self.__namespacePrefixCounter,)
            if not (candidate_prefix in self.__inScopeNamespaces):
                prefix = candidate_prefix
        ns = self.__inScopePrefixes.get(prefix)
        if ns:
            if ns != namespace:
                raise pyxb.LogicError('Prefix %s is already in use for %s' % (prefix, ns))
            return prefix
        if not self.__mutableInScopeNamespaces:
            self.__clonePrefixMap()
            self.__mutableInScopeNamespaces = True
        self.__addPrefixMap(prefix, namespace)
        return prefix

    def processXMLNS (self, prefix, uri):
        from pyxb.namespace import builtin
        if not self.__mutableInScopeNamespaces:
            self.__clonePrefixMap()
            self.__mutableInScopeNamespaces = True
        if builtin.XML.boundPrefix() == prefix:
            # Bound prefix xml is permitted if it's bound to the right URI, or
            # if the scope is being left.  In neither case is the mapping
            # adjusted.
            if (uri is None) or builtin.XML.uri() == uri:
                return
            raise pyxb.LogicError('Cannot manipulate bound prefix xml')
        if uri:
            if prefix is None:
                ns = self.__defaultNamespace = utility.NamespaceForURI(uri, create_if_missing=True)
                self.__inScopeNamespaces[None] = self.__defaultNamespace
            else:
                ns = utility.NamespaceForURI(uri, create_if_missing=True)
                self.__removePrefixMap(prefix)
                self.__addPrefixMap(prefix, ns)
            if self.__targetNamespace:
                self.__targetNamespace._referenceNamespace(ns)
            else:
                self.__pendingReferencedNamespaces.add(ns)
        else:
            # NB: XMLNS 6.2 says that you can undefine a default
            # namespace, but does not say anything explicitly about
            # undefining a prefixed namespace.  XML-Infoset 2.2
            # paragraph 6 implies you can do this, but expat blows up
            # if you try it.  I don't think it's legal.
            if prefix is not None:
                raise pyxb.NamespaceError(self, 'Attempt to undefine non-default namespace %s' % (prefix,))
            self.__removePrefixMap(prefix)
            self.__defaultNamespace = None

    def finalizeTargetNamespace (self, tns_uri=None, including_context=None):
        if tns_uri is not None:
            assert 0 < len(tns_uri)
            # Do not prevent overwriting target namespace; need this for WSDL
            # files where an embedded schema inadvertently inherits a target
            # namespace from its enclosing definitions element.  Note that if
            # we don't check this here, we do have to check it when schema
            # documents are included into parent schema documents.
            self.__targetNamespace = utility.NamespaceForURI(tns_uri, create_if_missing=True)
        elif self.__targetNamespace is None:
            if including_context is not None:
                self.__targetNamespace = including_context.targetNamespace()
                self.__fallbackToTargetNamespace = True
            elif tns_uri is None:
                self.__targetNamespace = utility.CreateAbsentNamespace()
            else:
                self.__targetNamespace = utility.NamespaceForURI(tns_uri, create_if_missing=True)
        if self.__pendingReferencedNamespaces is not None:
            [ self.__targetNamespace._referenceNamespace(_ns) for _ns in self.__pendingReferencedNamespaces ]
            self.__pendingReferencedNamespace = None
        assert self.__targetNamespace is not None
        if (not self.__fallbackToTargetNamespace) and self.__targetNamespace.isAbsentNamespace():
            self.__fallbackToTargetNamespace = True

    def reset (self):
        """Reset this instance to the state it was when created, exclusive of
        XMLNS directives passed in a constructor C{dom_node} parameter.

        This preserves parent context and constructor-specified prefix maps,
        but clears the namespace-prefix mapping of any additions made while
        processing namespace directives in DOM nodes, or manually added
        post-construction.

        The defaultNamespace is also retained."""
        self.__inScopeNamespaces = self.__initialScopeNamespaces
        self.__inScopePrefixes = self.__initialScopePrefixes
        self.__mutableInScopeNamespaces = False
        self.__namespacePrefixCounter = 0

    def __init__ (self,
                  dom_node=None,
                  parent_context=None,
                  including_context=None,
                  recurse=True,
                  default_namespace=None,
                  target_namespace=None,
                  in_scope_namespaces=None,
                  expanded_name=None,
                  finalize_target_namespace=True):  # MUST BE True for WSDL to work with minidom
        """Determine the namespace context that should be associated with the
        given node and, optionally, its element children.

        Primarily this class maintains a map between namespaces and prefixes
        used in QName instances.  The initial map comprises the bound prefixes
        (C{xml} and C{xmlns}), prefixes inherited from C{parent_context}, and
        prefixes passed through the C{in_scope_namespaces}
        parameter to the constructor.  This map is then augmented by any
        namespace declarations present in a passed C{dom_node}.  The initial
        map prior to augmentation may be restored through the L{reset()}
        method.

        @param dom_node: The DOM node
        @type dom_node: C{xml.dom.Element}
        @keyword parent_context: Optional value that specifies the context
        associated with C{dom_node}'s parent node.  If not provided, only the
        C{xml} namespace is in scope.
        @type parent_context: L{NamespaceContext}
        @keyword recurse: If True (default), create namespace contexts for all
        element children of C{dom_node}
        @type recurse: C{bool}
        @keyword default_namespace: Optional value to set as the default
        namespace.  Values from C{parent_context} would override this, as
        would an C{xmlns} attribute in the C{dom_node}.
        @type default_namespace: L{NamespaceContext}
        @keyword target_namespace: Optional value to set as the target
        namespace.  Values from C{parent_context} would override this, as
        would a C{targetNamespace} attribute in the C{dom_node}
        @type target_namespace: L{NamespaceContext}
        @keyword in_scope_namespaces: Optional value to set as the initial set
        of in-scope namespaces.  The always-present namespaces are added to
        this if necessary.
        @type in_scope_namespaces: C{dict} mapping prefix C{string} to L{Namespace}.
        """
        from pyxb.namespace import builtin

        if dom_node is not None:
            try:
                assert dom_node.__namespaceContext is None
            except AttributeError:
                pass
            dom_node.__namespaceContext = self

        self.__defaultNamespace = default_namespace
        self.__targetNamespace = target_namespace
        if self.__InitialScopeNamespaces is None:
            self.__BuildInitialPrefixMap()
        self.__inScopeNamespaces = self.__InitialScopeNamespaces
        self.__inScopePrefixes = self.__InitialScopePrefixes
        self.__mutableInScopeNamespaces = False
        self.__namespacePrefixCounter = 0

        if parent_context is not None:
            self.__inScopeNamespaces = parent_context.__inScopeNamespaces
            self.__inScopePrefixes = parent_context.__inScopePrefixes
            if parent_context.__mutableInScopeNamespaces:
                self.__clonePrefixMap()
            self.__defaultNamespace = parent_context.defaultNamespace()
            self.__targetNamespace = parent_context.targetNamespace()
            self.__fallbackToTargetNamespace = parent_context.__fallbackToTargetNamespace
        if in_scope_namespaces is not None:
            self.__clonePrefixMap()
            self.__mutableInScopeNamespaces = True
            for (pfx, ns) in six.iteritems(in_scope_namespaces):
                self.__removePrefixMap(pfx)
                self.__addPrefixMap(pfx, ns)

        # Record a copy of the initial mapping, exclusive of namespace
        # directives from C{dom_node}, so we can reset to that state.
        self.__initialScopeNamespaces = self.__inScopeNamespaces
        self.__initialScopePrefixes = self.__inScopePrefixes
        self.__mutableInScopeNamespaces = False

        if self.__targetNamespace is None:
            self.__pendingReferencedNamespaces = set()
        attribute_map = {}
        if dom_node is not None:
            if expanded_name is None:
                expanded_name = pyxb.namespace.ExpandedName(dom_node)
            for ai in range(dom_node.attributes.length):
                attr = dom_node.attributes.item(ai)
                if builtin.XMLNamespaces.uri() == attr.namespaceURI:
                    prefix = attr.localName
                    if 'xmlns' == prefix:
                        prefix = None
                    self.processXMLNS(prefix, attr.value)
                else:
                    if attr.namespaceURI is not None:
                        uri = utility.NamespaceForURI(attr.namespaceURI, create_if_missing=True)
                        key = pyxb.namespace.ExpandedName(uri, attr.localName)
                    else:
                        key = pyxb.namespace.ExpandedName(None, attr.localName)
                    attribute_map[key] = attr.value

        if finalize_target_namespace:
            tns_uri = None
            tns_attr = self._TargetNamespaceAttribute(expanded_name)
            if tns_attr is not None:
                tns_uri = attribute_map.get(tns_attr)
                self.finalizeTargetNamespace(tns_uri, including_context=including_context)

        # Store in each node the in-scope namespaces at that node;
        # we'll need them for QName interpretation of attribute
        # values.
        if (dom_node is not None) and recurse:
            from xml.dom import Node
            assert Node.ELEMENT_NODE == dom_node.nodeType
            for cn in dom_node.childNodes:
                if Node.ELEMENT_NODE == cn.nodeType:
                    NamespaceContext(dom_node=cn, parent_context=self, recurse=True)

    def interpretQName (self, name, namespace=None, default_no_namespace=False):
        """Convert the provided name into an L{ExpandedName}, i.e. a tuple of
        L{Namespace} and local name.

        If the name includes a prefix, that prefix must map to an in-scope
        namespace in this context.  Absence of a prefix maps to
        L{defaultNamespace()}, which must be provided (or defaults to the
        target namespace, if that is not absent).

        @param name: A QName.
        @type name: C{str} or C{unicode}
        @param name: Optional namespace to use for unqualified names when
        there is no default namespace.  Note that a defined default namespace,
        even if absent, supersedes this value.
        @keyword default_no_namespace: If C{False} (default), an NCName in a
        context where C{namespace} is C{None} and no default or fallback
        namespace can be identified produces an exception.  If C{True}, such an
        NCName is implicitly placed in no namespace.
        @return: An L{ExpandedName} tuple: ( L{Namespace}, C{str} )
        @raise pyxb.QNameResolutionError: The prefix is not in scope
        @raise pyxb.QNameResolutionError: No prefix is given and the default namespace is absent
        """
        if isinstance(name, pyxb.namespace.ExpandedName):
            return name
        assert isinstance(name, six.string_types)
        if 0 <= name.find(':'):
            (prefix, local_name) = name.split(':', 1)
            assert self.inScopeNamespaces() is not None
            namespace = self.inScopeNamespaces().get(prefix)
            if namespace is None:
                raise pyxb.QNameResolutionError('No namespace declaration for prefix', name, self)
        else:
            local_name = name
            # Context default supersedes caller-provided namespace
            if self.defaultNamespace() is not None:
                namespace = self.defaultNamespace()
            # If there's no default namespace, but there is a fallback
            # namespace, use that instead.
            if (namespace is None) and self.__fallbackToTargetNamespace:
                namespace = self.targetNamespace()
            if (namespace is None) and not default_no_namespace:
                raise pyxb.QNameResolutionError('NCName with no fallback/default namespace cannot be resolved', name, self)
        return pyxb.namespace.ExpandedName(namespace, local_name)

    def queueForResolution (self, component, depends_on=None):
        """Forwards to L{queueForResolution()<Namespace.queueForResolution>} in L{targetNamespace()}."""
        assert isinstance(component, _Resolvable_mixin)
        return self.targetNamespace().queueForResolution(component, depends_on)

## Local Variables:
## fill-column:78
## End:



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.utils.domutils
from pyxb.utils import six
from xml.dom import Node

import os.path
schema_path = os.path.abspath(os.path.join(os.path.dirname(__file__), '../schemas/particle.xsd'))
code = pyxb.binding.generate.GeneratePython(schema_location=schema_path)
rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

from pyxb.utils import domutils
def ToDOM (instance, tag=None):
    return instance.toDOM().documentElement

class TestParticle (unittest.TestCase):
    def test_bad_creation (self):
        xmlt = six.u('<h01 xmlns="URN:test"/>')
        dom = pyxb.utils.domutils.StringToDOM(xmlt)
        # Creating with wrong element
        self.assertRaises(pyxb.StructuralBadDocumentError, h01b.createFromDOM, dom.documentElement)

    def test_h01_empty (self):
        xmlt = six.u('<ns1:h01 xmlns:ns1="URN:test"/>')
        xmld = xmlt.encode('utf-8')
        dom = pyxb.utils.domutils.StringToDOM(xmlt)
        instance = h01.createFromDOM(dom.documentElement)
        self.assertTrue(instance.elt is None)
        self.assertEqual(ToDOM(instance).toxml("utf-8"), xmld)

    def test_h01_elt (self):
        xmlt = six.u('<ns1:h01 xmlns:ns1="URN:test"><elt/></ns1:h01>')
        xmld = xmlt.encode('utf-8')
        dom = pyxb.utils.domutils.StringToDOM(xmlt)
        instance = h01.createFromDOM(dom.documentElement)
        self.assertTrue(instance.elt is not None)
        self.assertEqual(ToDOM(instance).toxml("utf-8"), xmld)

    def test_h01_elt2 (self):
        xmlt = six.u('<h01 xmlns="URN:test"><elt/><elt/></h01>')
        dom = pyxb.utils.domutils.StringToDOM(xmlt)
        self.assertRaises(UnrecognizedContentError, h01.createFromDOM, dom.documentElement)

    def test_h01b_empty (self):
        xmlt = six.u('<ns1:h01b xmlns:ns1="URN:test"/>')
        xmld = xmlt.encode('utf-8')
        dom = pyxb.utils.domutils.StringToDOM(xmlt)
        instance = h01b.createFromDOM(dom.documentElement)
        self.assertTrue(instance.elt is None)
        self.assertEqual(ToDOM(instance).toxml("utf-8"), xmld)

    def test_h01b_elt (self):
        xmlt = six.u('<ns1:h01b xmlns:ns1="URN:test"><elt/></ns1:h01b>')
        xmld = xmlt.encode('utf-8')
        dom = pyxb.utils.domutils.StringToDOM(xmlt)
        instance = h01b.createFromDOM(dom.documentElement)
        self.assertTrue(instance.elt is not None)
        self.assertEqual(ToDOM(instance).toxml("utf-8"), xmld)

    def test_h01b_elt2 (self):
        xmlt = six.u('<ns1:h01b xmlns:ns1="URN:test"><elt/><elt/></ns1:h01b>')
        dom = pyxb.utils.domutils.StringToDOM(xmlt)
        self.assertRaises(UnrecognizedContentError, h01b.createFromDOM, dom.documentElement)

    def test_h11_empty (self):
        xmlt = six.u('<ns1:h11 xmlns:ns1="URN:test"/>')
        dom = pyxb.utils.domutils.StringToDOM(xmlt)
        self.assertRaises(IncompleteElementContentError, h11.createFromDOM, dom.documentElement)

    def test_h11_elt (self):
        xmlt = six.u('<ns1:h11 xmlns:ns1="URN:test"><elt/></ns1:h11>')
        xmld = xmlt.encode('utf-8')
        dom = pyxb.utils.domutils.StringToDOM(xmlt)
        instance = h11.createFromDOM(dom.documentElement)
        self.assertTrue(instance.elt is not None)
        self.assertEqual(ToDOM(instance).toxml("utf-8"), xmld)

    def test_h24 (self):
        xmlt = six.u('<h24 xmlns="URN:test"></h24>')
        dom = pyxb.utils.domutils.StringToDOM(xmlt)
        self.assertRaises(IncompleteElementContentError, h24.createFromDOM, dom.documentElement)

        for num_elt in range(0, 5):
            xmlt = six.u('<ns1:h24 xmlns:ns1="URN:test">%s</ns1:h24>') % (''.join(num_elt * ['<elt/>']),)
            dom = pyxb.utils.domutils.StringToDOM(xmlt)
            if 2 > num_elt:
                self.assertRaises(IncompleteElementContentError, h24.createFromDOM, dom.documentElement)
            elif 4 >= num_elt:
                instance = h24.createFromDOM(dom.documentElement)
                xmld = xmlt.encode('utf-8')
                self.assertEqual(num_elt, len(instance.elt))
                self.assertEqual(ToDOM(instance).toxml("utf-8"), xmld)
            else:
                self.assertRaises(UnrecognizedContentError, h24.createFromDOM, dom.documentElement)

    def test_h24b (self):
        xmlt = six.u('<ns1:h24b xmlns:ns1="URN:test"></ns1:h24b>')
        dom = pyxb.utils.domutils.StringToDOM(xmlt)
        self.assertRaises(IncompleteElementContentError, h24b.createFromDOM, dom.documentElement)

        for num_elt in range(0, 5):
            xmlt = six.u('<ns1:h24b xmlns:ns1="URN:test">%s</ns1:h24b>') % (''.join(num_elt * ['<elt/>']),)
            dom = pyxb.utils.domutils.StringToDOM(xmlt)
            if 2 > num_elt:
                self.assertRaises(IncompleteElementContentError, h24b.createFromDOM, dom.documentElement)
            elif 4 >= num_elt:
                xmld = xmlt.encode('utf-8')
                instance = h24b.createFromDOM(dom.documentElement)
                self.assertEqual(num_elt, len(instance.elt))
                self.assertEqual(ToDOM(instance).toxml("utf-8"), xmld)
            else:
                self.assertRaises(UnrecognizedContentError, h24b.createFromDOM, dom.documentElement)

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.utils.domutils
from xml.dom import Node

import os.path
schema_path = os.path.abspath(os.path.join(os.path.dirname(__file__), '../schemas/test-union.xsd'))
code = pyxb.binding.generate.GeneratePython(schema_location=schema_path)
#open('code.py', 'w').write(code)
rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

class TestUnion (unittest.TestCase):
    def test (self):
        self.assertRaises(LogicError, myUnion, 5)
        self.assertEqual(5, myUnion.Factory(5))
        self.assertEqual(5, myUnion.Factory('5'))
        self.assertRaises(SimpleTypeValueError, myUnion.Factory, 10)
        self.assertTrue(isinstance(myUnion.Factory('5'), singleDigit))
        self.assertTrue(isinstance(myUnion.Factory('one'), english))
        self.assertEqual(welsh.un, myUnion.Factory('un'))
        self.assertTrue(isinstance(myUnion.Factory('un'), welsh))
        self.assertEqual(english.one, myUnion.Factory('one'))
        self.assertRaises(LogicError, myUnion, 'five')

    def testList (self):
        my_list = unionList([ myUnion.Factory(4), myUnion.Factory('one')])
        self.assertEqual(2, len(my_list))

    def testRestrictedUnion (self):
        self.assertEqual(ones.one, ones.Factory('one'))
        self.assertRaises(SimpleTypeValueError, ones.Factory, 'two')
        self.assertEqual(ones.un, ones.Factory('un'))

    def testAnonymousUnion (self):
        self.assertEqual('four', words2.Factory('four'))
        self.assertEqual('pump', words2.Factory('pump'))
        self.assertRaises(SimpleTypeValueError,  words2.Factory, 'one')

    def testFiveWords (self):
        self.assertEqual('one', fiveWords.Factory('one'))
        self.assertEqual('dau', fiveWords.Factory('dau'))
        self.assertEqual('four', fiveWords.Factory('four'))
        self.assertEqual('pump', fiveWords.Factory('pump'))

    def testMyElement (self):
        self.assertEqual(0, myElement('0'))
        self.assertEqual(english.two, myElement('two'))
        self.assertEqual(welsh.tri, myElement('tri'))
        self.assertRaises(SimpleTypeValueError, myElement, 'five')

    def testValidation (self):
        # Test automated conversion
        uv = myUnion._ValidatedMember('one')
        self.assertTrue(isinstance(uv, english))
        uv = myUnion._ValidatedMember('tri')
        self.assertTrue(isinstance(uv, welsh))

    def testXsdLiteral (self):
        ul = unionList([0, 'un', 'one'])
        self.assertEqual('0 un one', ul.xsdLiteral())

    def testXMLErrors (self):
        self.assertEqual(welsh.un, CreateFromDocument('<myElement xmlns="URN:unionTest">un</myElement>'))
        self.assertRaises(UnrecognizedDOMRootNodeError, CreateFromDocument, '<welsh>un</welsh>')
        self.assertRaises(UnrecognizedDOMRootNodeError, CreateFromDocument, '<myelement>un</myelement>')

    def testMyElementDOM (self):
        self.assertEqual(0, myElement.createFromDOM(pyxb.utils.domutils.StringToDOM('<myElement xmlns="URN:unionTest">0</myElement>').documentElement))
        self.assertEqual(0, CreateFromDocument('<myElement xmlns="URN:unionTest">0</myElement>'))

        self.assertEqual(english.one, myElement.createFromDOM(pyxb.utils.domutils.StringToDOM('<myElement xmlns="URN:unionTest">one</myElement>').documentElement))
        self.assertEqual(welsh.un, myElement.createFromDOM(pyxb.utils.domutils.StringToDOM('<myElement xmlns="URN:unionTest">un</myElement>').documentElement))

        self.assertEqual(english.one, myElement.createFromDOM(pyxb.utils.domutils.StringToDOM('<myElement xmlns="URN:unionTest">one<!-- with comment --></myElement>').documentElement))
        self.assertEqual(welsh.un, myElement.createFromDOM(pyxb.utils.domutils.StringToDOM('<myElement xmlns="URN:unionTest"><!-- with comment -->un</myElement>').documentElement))

        self.assertEqual(english.one, myElement.createFromDOM(pyxb.utils.domutils.StringToDOM('<myElement xmlns="URN:unionTest"> one <!-- with comment and whitespace --></myElement>').documentElement))
        self.assertRaises(SimpleTypeValueError, myElement.createFromDOM, pyxb.utils.domutils.StringToDOM('<myElement xmlns="URN:unionTest"><!-- whitespace is error for welsh --> un</myElement>').documentElement)

        self.assertEqual(english.one, myElement.createFromDOM(pyxb.utils.domutils.StringToDOM('''<myElement xmlns="URN:unionTest"><!-- whitespace is collapsed for english -->
one
</myElement>''').documentElement))
        self.assertRaises(SimpleTypeValueError, myElement.createFromDOM, pyxb.utils.domutils.StringToDOM('''<myElement xmlns="URN:unionTest"><!--whitespace is only reduced for welsh -->
un
</myElement>''').documentElement)

if __name__ == '__main__':
    unittest.main()
